/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEIB
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2016 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file EucalyptusParameter.hpp
 * @brief 
 *
 *
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision$
 * $Date$
 * Last modified by $Author$
 *
*/
#ifndef EUCALYPTUSPARAMETER_HPP
#define EUCALYPTUSPARAMETER_HPP

#include "Parameter.hpp"
#include "refcount.hpp"

/**
 * @name Forward Declarations
 */
//@{
REF_FORWARD_DECL(EucalyptusParameter);
//@}

class EucalyptusParameter : public Parameter
{
   private:
      /**
       * Check the compatibility among the different parameters
       * and compute implicated parameters
       */
      void CheckParameters();

      /**
       * Sets the default values with respect to the tool
       */
      void SetDefaults();

      /**
       * Print the usage of this tool
       * @param os is the stream where the message has to be printed
       */
      void PrintHelp(std::ostream &os) const;

      /**
       * Print the name of the program to be included in the header
       * @param os is the stream on which the program name has to be printed
       */
      void PrintProgramName(std::ostream & os) const;

   public:
      /**
       * Default Constructor. It sets default values
       * @param program_name is the name of the executable
       */
      EucalyptusParameter(const std::string program_name);

      /**
       * Copy Constructor. It sets default values and then copies (overwrites) the parameters stored in the provided datastructure
       * @param Param is the datastructure where the parameters to be copied are stored
       */
      EucalyptusParameter(const Parameter & Param);

      /**
       * Destructor
       */
      virtual ~EucalyptusParameter() {}

      /**
       * Execute parameter parsing
       * @param argc is the number of arguments
       * @param argv is the array of arguments given to program.
       */
      virtual int exec(int argc, char * const argv[]);
};

typedef refcount<EucalyptusParameter> EucalyptusParameterRef;

#endif
