/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEIB
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2016 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file HLS_flow_base_step.hpp
 * @brief This flow is used to generate the synthesis of the input specification
 *
 * This flow generates at least an implementation point for each module, subjected to some design constraints.
 * It can include global or local synthesis steps
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision$
 * $Date$
 * Last modified by $Author$
 *
 */
#ifndef HLS_FLOW_BASE_STEP_HPP
#define HLS_FLOW_BASE_STEP_HPP

///Superclass include
#include "hls_function_step.hpp"

class HLSFlowBaseStep : public HLSFunctionStep
{
   public:
      /**
       * Constructor
       * @param Param is the se of input parameters
       * @param HLSMgr is the HLS manager
       * @param function_index is the function index
       * @param design_flow_manager is the design flow manager
       * @param HLSFlowStep_Type is the type of hls synthesis flow to be created
       */
      HLSFlowBaseStep(const ParameterConstRef Param, const HLS_managerRef HLSMgr, const unsigned int function_index, const DesignFlowManagerConstRef design_flow_manager, const HLSFlowStep_Type hls_flow_step_type);

      /**
       * Destructor
       */
      virtual ~HLSFlowBaseStep();
};
#endif
