/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEIB
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2016 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file module_interface.cpp
 * @brief Base class for all module interfaces
 *
 *
 * @author Christian Pilato <pilato@elet.polimi.it>
 * $Revision$
 * $Date$
 * Last modified by $Author$
 * $Locker:  $
 * $State: Exp $
 *
*/
///Autoheader include
#include "config_HAVE_EXPERIMENTAL.hpp"

#include "module_interface.hpp"

#include "call_graph_manager.hpp"
#include "function_behavior.hpp"
#include "behavioral_helper.hpp"

#include "hls.hpp"
#include "hls_manager.hpp"
#include "memory.hpp"
#include "memory_symbol.hpp"

#include "structural_manager.hpp"
#include "structural_objects.hpp"

#include "polixml.hpp"
#include "xml_helper.hpp"

#include "Parameter.hpp"
#include "constant_strings.hpp"

///HLS/module_allocation includes
#include "add_library.hpp"

module_interface::module_interface(const ParameterConstRef _parameters, const HLS_managerRef _HLSMgr, unsigned int _funId, const DesignFlowManagerConstRef _design_flow_manager, const HLSFlowStep_Type _hls_flow_step_type) :
   HLSFunctionStep(_parameters, _HLSMgr, _funId, _design_flow_manager, _hls_flow_step_type)
{
}

module_interface::~module_interface()
{
}

const std::unordered_set<std::tuple<HLSFlowStep_Type, HLSFlowStepSpecializationConstRef, HLSFlowStep_Relationship> > module_interface::ComputeHLSRelationships(const DesignFlowStep::RelationshipType relationship_type) const
{
   std::unordered_set<std::tuple<HLSFlowStep_Type, HLSFlowStepSpecializationConstRef, HLSFlowStep_Relationship> > ret;
   switch(relationship_type)
   {
      case DEPENDENCE_RELATIONSHIP:
         {
            if (HLSMgr->hasToBeInterfaced(funId) and (HLSMgr->existAdditionalTop() or hls_flow_step_type == HLSFlowStep_Type::WB4_INTERFACE_GENERATION))
            {
               ret.insert(std::make_tuple(HLSFlowStep_Type::TOP_ENTITY_MEMORY_MAPPED_CREATION,
                                          HLSFlowStepSpecializationConstRef(),
                                          HLSFlowStep_Relationship::SAME_FUNCTION));
            }
            else
            {
               ret.insert(std::make_tuple(HLSFlowStep_Type::TOP_ENTITY_CREATION,
                                          HLSFlowStepSpecializationConstRef(),
                                          HLSFlowStep_Relationship::SAME_FUNCTION));
            }
            break;
         }
      case INVALIDATION_RELATIONSHIP:
         {
            break;
         }
      case PRECEDENCE_RELATIONSHIP:
         {
            ret.insert(std::make_tuple(HLSFlowStep_Type::ADD_LIBRARY, HLSFlowStepSpecializationConstRef(new AddLibrarySpecialization(false)), HLSFlowStep_Relationship::SAME_FUNCTION));
            break;
         }
      default:
         THROW_UNREACHABLE("");
   }
   return ret;
}

void module_interface::add_sign(const structural_managerRef SM, const structural_objectRef sig1, const structural_objectRef sig2, const std::string& sig_name)
{
   structural_objectRef sig = SM->add_sign(sig_name, SM->get_circ(), sig1->get_typeRef());
   SM->add_connection(sig, sig1);
   SM->add_connection(sig, sig2);
}

void module_interface::add_sign_vector(const structural_managerRef SM, const structural_objectRef sig1, const structural_objectRef sig2, const std::string& sig_name)
{
   THROW_ASSERT(GetPointer<port_o>(sig1)->get_ports_size(), "");
   structural_objectRef sig = SM->add_sign_vector(sig_name, GetPointer<port_o>(sig1)->get_ports_size(), SM->get_circ(), sig1->get_typeRef());
   SM->add_connection(sig, sig1);
   SM->add_connection(sig, sig2);
}
