/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEIB
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2016 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file hls_bit_value.cpp
 * @brief Wrapper for bit value analysis in the HLS context
 *
 * @author Fabrizio Ferrandi <fabrizio.ferrandi@polimi.it>
 * $Revision$
 * $Date$
 * Last modified by $Author$
 *
*/

#include "hls_bit_value.hpp"

#include "hls.hpp"
#include "utility.hpp"
#include "exceptions.hpp"
#include "utility.hpp"
#include "hls_manager.hpp"
#include "function_behavior.hpp"
#include "behavioral_helper.hpp"


#include "design_flow_manager.hpp"
#include "frontend_flow_step_factory.hpp"
#include "frontend_flow_step.hpp"

#include <boost/lexical_cast.hpp>
#include <boost/algorithm/string.hpp>

#include <map>
#include <vector>
#include <cmath>

#include <iostream>

#include "Parameter.hpp"
#include "dbgPrintHelper.hpp"

///HLS includes
#include "hls_flow_step_factory.hpp"


hls_bit_value::hls_bit_value(const ParameterConstRef _parameters, const HLS_managerRef _HLSMgr, unsigned _funId, const DesignFlowManagerConstRef _design_flow_manager):
      HLSFunctionStep(_parameters, _HLSMgr, _funId, _design_flow_manager, HLSFlowStep_Type::HLS_BIT_VALUE, HLSFlowStepSpecializationConstRef())
{
   debug_level = _parameters->get_class_debug_level(GET_CLASS(*this));
}

hls_bit_value::~hls_bit_value()
{
}

void hls_bit_value::Initialize()
{
   HLSFunctionStep::Initialize();
}

const std::unordered_set<std::tuple<HLSFlowStep_Type, HLSFlowStepSpecializationConstRef, HLSFlowStep_Relationship> > hls_bit_value::ComputeHLSRelationships(const DesignFlowStep::RelationshipType relationship_type) const
{
   std::unordered_set<std::tuple<HLSFlowStep_Type, HLSFlowStepSpecializationConstRef, HLSFlowStep_Relationship> > ret;
   switch(relationship_type)
   {
      case DEPENDENCE_RELATIONSHIP:
         {
            ret.insert(std::make_tuple(HLSFlowStep_Type::HLS_SYNTHESIS_FLOW, HLSFlowStepSpecializationConstRef(), HLSFlowStep_Relationship::CALLED_FUNCTIONS));
            ret.insert(std::make_tuple(HLSFlowStep_Type::INITIALIZE_HLS, HLSFlowStepSpecializationConstRef(), HLSFlowStep_Relationship::SAME_FUNCTION));
            break;
         }
      case INVALIDATION_RELATIONSHIP:
         {
            break;
         }
      case PRECEDENCE_RELATIONSHIP:
         {
            break;
         }
      default:
         THROW_UNREACHABLE("");
   }
   return ret;
}



DesignFlowStep_Status hls_bit_value::InternalExec()
{
///   if(output_level == OUTPUT_LEVEL_VERBOSE or output_level == OUTPUT_LEVEL_PEDANTIC)
///      INDENT_OUT_MEX(OUTPUT_LEVEL_VERBOSE, output_level, "");
///   INDENT_OUT_MEX(OUTPUT_LEVEL_VERBOSE, output_level, "-->Bit Value analysis for function " + HLS->FB->CGetBehavioralHelper()->get_function_name());
   unsigned int curr_address_bitsize = HLSMgr->get_address_bitsize();
   unsigned int default_address_bitsize = parameters->isOption(OPT_addr_bus_bitsize) ? parameters->getOption<unsigned int>(OPT_addr_bus_bitsize) : 32;
   if(default_address_bitsize != curr_address_bitsize)
   {
///      INDENT_OUT_MEX(OUTPUT_LEVEL_VERBOSE, output_level, "-->Restart Bit Value analysis for function " + HLS->FB->CGetBehavioralHelper()->get_function_name());
      const DesignFlowStepRef design_flow_step = GetPointer<const FrontendFlowStepFactory>(design_flow_manager.lock()->CGetDesignFlowStepFactory("Frontend"))->CreateFunctionFrontendFlowStep(FrontendFlowStepType::BIT_VALUE, funId);
      design_flow_step->Initialize();
      const DesignFlowStep_Status return_status = design_flow_step->Exec();
      if(return_status != DesignFlowStep_Status::SUCCESS)
         return return_status;
///      INDENT_OUT_MEX(OUTPUT_LEVEL_VERBOSE, output_level, "<--");
   }
///   INDENT_OUT_MEX(OUTPUT_LEVEL_VERBOSE, output_level, "<--");
///   if(output_level == OUTPUT_LEVEL_VERBOSE or output_level == OUTPUT_LEVEL_PEDANTIC)
///      INDENT_OUT_MEX(OUTPUT_LEVEL_VERBOSE, output_level, "");
   return DesignFlowStep_Status::SUCCESS;
}
