/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEIB
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2016 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file xml_node.cpp
 * @brief
 *
 * @author Fabrizio Ferrandi <fabrizio.ferrandi@polimi.it>
 * $Revision$
 * $Date$
 * Last modified by $Author$
 *
 */

#include "xml_node.hpp"
#include "xml_element.hpp"
#include "xml_text_node.hpp"
#include "xml_comment_node.hpp"
#include "xml_att_decl_node.hpp"

xml_element* xml_child::add_child_element(const std::string & _name)
{
   xml_element* new_el = new xml_element(_name);
   xml_nodeRef new_ref(new_el);
   child_list.push_back(new_ref);
   return new_el;
}

xml_element* xml_child::add_child_element(const xml_nodeRef& node)
{
   child_list.push_back(node);
   return GetPointer<xml_element>(node);
}


xml_text_node* xml_child::add_child_text(const std::string& content)
{
   xml_text_node* new_el = new xml_text_node(content);
   if(!first_text)
      first_text = new_el;
   xml_nodeRef new_ref(new_el);
   child_list.push_back(new_ref);
   return new_el;
}

xml_comment_node* xml_child::add_child_comment(const std::string& content)
{
   xml_comment_node* new_el = new xml_comment_node(content);
   xml_nodeRef new_ref(new_el);
   child_list.push_back(new_ref);
   return new_el;
}

xml_att_decl_node* xml_child::add_child_attribute_declaration(const std::string& _name)
{
   xml_att_decl_node* new_el = new xml_att_decl_node(_name);
   xml_nodeRef new_ref(new_el);
   child_list.push_back(new_ref);
   return new_el;
}

void xml_node::set_line(int _line) 
{
   line = _line;
}

int xml_node::get_line() const
{
   return line;
}
