/*
 *
 *                   _/_/_/    _/_/   _/    _/ _/_/_/    _/_/
 *                  _/   _/ _/    _/ _/_/  _/ _/   _/ _/    _/
 *                 _/_/_/  _/_/_/_/ _/  _/_/ _/   _/ _/_/_/_/
 *                _/      _/    _/ _/    _/ _/   _/ _/    _/
 *               _/      _/    _/ _/    _/ _/_/_/  _/    _/
 *
 *             ***********************************************
 *                              PandA Project 
 *                     URL: http://panda.dei.polimi.it
 *                       Politecnico di Milano - DEIB
 *                        System Architectures Group
 *             ***********************************************
 *              Copyright (c) 2004-2016 Politecnico di Milano
 *
 *   This file is part of the PandA framework.
 *
 *   The PandA framework is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
/**
 * @file tree-panda-gcc-Parameter.hpp
 *
 * @author Fabrizio Ferrandi <fabrizio.ferrandi@polimi.it>
 * $Revision$
 * $Date$
 * Last modified by $Author$
 *
*/
#ifndef TREE_PANDA_GCC_PARAMETER_HPP
#define TREE_PANDA_GCC_PARAMETER_HPP

///Superclass include
#include "Parameter.hpp"

///Utility include
#include "refcount.hpp"

class tree_panda_gcc_parameter : public Parameter
{
   private:
      /**
       * Checks parameter values to set implicit one
       */
      virtual void CheckParameters();

      /**
       * Print the usage of this tool
       * @param os is the stream where the message has to be printed
       */
      void PrintHelp(std::ostream &os) const;

      /**
       * Print the name of the program to be included in the header
       * @param os is the stream on which the program name has to be printed
       */
      void PrintProgramName(std::ostream & os) const;

   public:
      /**
       * Constructor
       * @param program_name
       */
      tree_panda_gcc_parameter(const std::string program_name);

      /**
       * Destructor
       */
      virtual ~tree_panda_gcc_parameter();

      /**
       * Execute parameter parsing
       * @param argc is the number of arguments
       * @param argv is the array of arguments passed to program.
       */
      virtual int exec(int argc, char * const argv[]);

      /**
       * Sets the default values for the Zebu tool
       */
      void SetDefaults();
};

#endif
