/*
+--------------------------------------------------------------------------+
| CHStone : a suite of benchmark programs for C-based High-Level Synthesis |
| ======================================================================== |
|                                                                          |
| * Collected and Modified : Y. Hara, H. Tomiyama, S. Honda,               |
|                            H. Takada and K. Ishii                        |
|                            Nagoya University, Japan                      |
|                                                                          |
| * Remark :                                                               |
|    1. This source code is modified to unify the formats of the benchmark |
|       programs in CHStone.                                               |
|    2. Test vectors are added for CHStone.                                |
|    3. If "main_result" is 0 at the end of the program, the program is    |
|       correctly executed.                                                |
|    4. Please follow the copyright of each benchmark program.             |
+--------------------------------------------------------------------------+
*/
/*
 * Copyright (C) 2008
 * Y. Hara, H. Tomiyama, S. Honda, H. Takada and K. Ishii
 * Nagoya University, Japan
 * All rights reserved.
 *
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis. The authors disclaims  any and all warranties, 
 * whether express, implied, or statuary, including any implied warranties or 
 * merchantability or of fitness for a particular purpose. In no event shall the
 * copyright-holder be liable for any incidental, punitive, or consequential damages
 * of any kind whatsoever arising from the use of these programs. This disclaimer
 * of warranty extends to the user of these programs and user's customers, employees,
 * agents, transferees, successors, and assigns.
 *
 */
int main_result;
/*
 * Output Buffer
 */
unsigned char *CurHuffReadBuf;
int OutData_image_width;
int OutData_image_height;
int OutData_comp_vpos[RGB_NUM];
int OutData_comp_hpos[RGB_NUM];
unsigned char OutData_comp_buf[RGB_NUM][BMP_OUT_SIZE];

/*
+--------------------------------------------------------------------------+
| * Test Vector (added for CHStone)                                        |
|     hana_jpg : input data                                                |
|     hana_bmp : expected output data                                      |
|     out_width : expected output data                                     |
|     out_length : expected output data                                    |
+--------------------------------------------------------------------------+
*/
#define JPEGSIZE 5207

const unsigned char hana_jpg[JPEGSIZE] = {
    255, 216, 255, 224, 0, 16, 74, 70, 73, 70, 0, 1, 1, 0, 0, 1, 0, 1, 0, 0,
    255, 219, 0, 67, 0, 3, 2, 2, 2, 2, 2, 3, 2, 2, 2, 3, 3, 3, 3, 4, 6, 4, 4,
    4, 4, 4, 8, 6, 6, 5, 6, 9, 8, 10, 10, 9, 8, 9, 9, 10, 12, 15, 12, 10, 11,
    14, 11, 9, 9, 13, 17, 13, 14, 15, 16, 16, 17, 16, 10, 12, 18, 19, 18, 16,
    19, 15, 16, 16, 16, 255, 219, 0, 67, 1, 3, 3, 3, 4, 3, 4, 8, 4, 4, 8, 16,
    11, 9, 11, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16,
    16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16,
    16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 255, 192,
    0, 17, 8, 0, 59, 0, 90, 3, 1, 34, 0, 2, 17, 1, 3, 17, 1, 255, 196, 0, 31,
    0, 0, 1, 5, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 3, 4, 5, 6, 7,
    8, 9, 10, 11, 255, 196, 0, 181, 16, 0, 2, 1, 3, 3, 2, 4, 3, 5, 5, 4, 4, 0,
    0, 1, 125, 1, 2, 3, 0, 4, 17, 5, 18, 33, 49, 65, 6, 19, 81, 97, 7, 34,
    113, 20, 50, 129, 145, 161, 8, 35, 66, 177, 193, 21, 82, 209, 240, 36, 51,
    98, 114, 130, 9, 10, 22, 23, 24, 25, 26, 37, 38, 39, 40, 41, 42, 52, 53,
    54, 55, 56, 57, 58, 67, 68, 69, 70, 71, 72, 73, 74, 83, 84, 85, 86, 87,
    88, 89, 90, 99, 100, 101, 102, 103, 104, 105, 106, 115, 116, 117, 118,
    119, 120, 121, 122, 131, 132, 133, 134, 135, 136, 137, 138, 146, 147, 148,
    149, 150, 151, 152, 153, 154, 162, 163, 164, 165, 166, 167, 168, 169, 170,
    178, 179, 180, 181, 182, 183, 184, 185, 186, 194, 195, 196, 197, 198, 199,
    200, 201, 202, 210, 211, 212, 213, 214, 215, 216, 217, 218, 225, 226, 227,
    228, 229, 230, 231, 232, 233, 234, 241, 242, 243, 244, 245, 246, 247, 248,
    249, 250, 255, 196, 0, 31, 1, 0, 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0,
    0, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 255, 196, 0, 181, 17, 0, 2, 1, 2,
    4, 4, 3, 4, 7, 5, 4, 4, 0, 1, 2, 119, 0, 1, 2, 3, 17, 4, 5, 33, 49, 6, 18,
    65, 81, 7, 97, 113, 19, 34, 50, 129, 8, 20, 66, 145, 161, 177, 193, 9, 35,
    51, 82, 240, 21, 98, 114, 209, 10, 22, 36, 52, 225, 37, 241, 23, 24, 25,
    26, 38, 39, 40, 41, 42, 53, 54, 55, 56, 57, 58, 67, 68, 69, 70, 71, 72,
    73, 74, 83, 84, 85, 86, 87, 88, 89, 90, 99, 100, 101, 102, 103, 104, 105,
    106, 115, 116, 117, 118, 119, 120, 121, 122, 130, 131, 132, 133, 134, 135,
    136, 137, 138, 146, 147, 148, 149, 150, 151, 152, 153, 154, 162, 163, 164,
    165, 166, 167, 168, 169, 170, 178, 179, 180, 181, 182, 183, 184, 185, 186,
    194, 195, 196, 197, 198, 199, 200, 201, 202, 210, 211, 212, 213, 214, 215,
    216, 217, 218, 226, 227, 228, 229, 230, 231, 232, 233, 234, 242, 243, 244,
    245, 246, 247, 248, 249, 250, 255, 218, 0, 12, 3, 1, 0, 2, 17, 3, 17, 0,
    63, 0, 245, 175, 0, 90, 234, 23, 179, 175, 138, 60, 79, 49, 241, 164, 122,
    155, 207, 169, 233, 190, 34, 176, 103, 22, 122, 108, 86, 241, 42, 91, 218,
    172, 12, 73, 44, 199, 204, 38, 64, 25, 100, 44, 120, 227, 11, 192, 232,
    186, 55, 237, 59, 163, 77, 227, 77, 59, 91, 241, 127, 133, 53, 77, 6, 210,
    214, 226, 246, 59, 184, 68, 240, 221, 88, 75, 114, 4, 136, 51, 144, 86,
    40, 144, 60, 152, 44, 85, 190, 81, 236, 191, 42, 120, 51, 246, 139, 241,
    15, 194, 95, 136, 109, 164, 69, 172, 234, 103, 194, 209, 222, 44, 113,
    218, 195, 117, 152, 35, 182, 145, 140, 140, 33, 141, 183, 4, 67, 191, 32,
    32, 0, 2, 112, 58, 87, 176, 248, 23, 246, 198, 240, 192, 241, 97, 241, 71,
    136, 252, 49, 44, 26, 191, 142, 174, 35, 182, 186, 213, 18, 255, 0, 203,
    182, 210, 116, 245, 142, 59, 104, 164, 114, 220, 200, 4, 176, 179, 72, 89,
    64, 10, 196, 142, 165, 31, 183, 13, 142, 165, 207, 25, 41, 107, 54, 155,
    190, 154, 53, 183, 158, 189, 254, 243, 23, 36, 210, 232, 125, 11, 169,
    106, 22, 30, 53, 248, 24, 38, 241, 78, 129, 164, 120, 150, 77, 70, 218,
    59, 233, 45, 116, 235, 105, 239, 237, 117, 11, 121, 68, 91, 175, 227, 138,
    117, 142, 72, 144, 13, 211, 24, 147, 44, 129, 65, 82, 225, 129, 63, 40,
    248, 147, 246, 97, 240, 171, 147, 166, 232, 207, 172, 75, 127, 115, 49,
    107, 59, 171, 11, 121, 47, 116, 187, 168, 229, 93, 233, 17, 92, 151, 183,
    117, 31, 46, 114, 193, 128, 29, 73, 192, 250, 171, 197, 158, 10, 186, 215,
    190, 201, 27, 248, 170, 214, 238, 65, 99, 117, 53, 190, 149, 103, 107, 29,
    197, 148, 147, 136, 213, 51, 5, 199, 221, 183, 60, 67, 34, 146, 225, 136,
    50, 47, 40, 94, 188, 39, 225, 135, 197, 79, 140, 176, 124, 71, 130, 211,
    196, 99, 86, 159, 69, 180, 105, 45, 102, 210, 172, 109, 18, 91, 149, 115,
    132, 134, 109, 235, 242, 172, 70, 86, 96, 29, 159, 14, 85, 143, 64, 113,
    234, 202, 24, 90, 139, 146, 178, 77, 108, 180, 244, 210, 253, 29, 255, 0,
    34, 188, 217, 243, 79, 194, 205, 67, 199, 31, 3, 190, 37, 233, 94, 43,
    131, 69, 125, 40, 67, 168, 191, 135, 239, 110, 174, 173, 159, 236, 241,
    201, 36, 138, 178, 197, 185, 134, 3, 170, 236, 111, 80, 59, 87, 214, 31,
    181, 100, 43, 96, 154, 69, 151, 136, 252, 9, 160, 155, 93, 80, 197, 168,
    182, 165, 22, 158, 12, 223, 104, 82, 67, 66, 210, 224, 97, 242, 1, 32,
    158, 71, 110, 227, 99, 225, 111, 195, 47, 9, 124, 69, 182, 241, 47, 138,
    124, 117, 62, 191, 110, 214, 126, 54, 215, 230, 184, 133, 84, 186, 49,
    105, 150, 35, 148, 25, 141, 220, 40, 100, 37, 121, 4, 177, 7, 28, 142,
    183, 195, 254, 40, 93, 65, 109, 62, 10, 124, 74, 181, 212, 181, 211, 169,
    217, 223, 237, 212, 46, 237, 161, 49, 106, 145, 197, 181, 68, 222, 100,
    96, 32, 102, 89, 16, 2, 131, 118, 118, 147, 142, 181, 231, 82, 192, 202,
    174, 14, 88, 106, 83, 183, 58, 222, 215, 181, 250, 124, 204, 181, 148, 79,
    132, 60, 73, 6, 179, 172, 71, 125, 171, 92, 218, 172, 150, 146, 48, 54,
    168, 23, 228, 183, 128, 28, 2, 61, 115, 236, 59, 241, 197, 105, 124, 33,
    248, 7, 226, 175, 142, 26, 243, 190, 157, 172, 219, 218, 195, 164, 95, 90,
    166, 161, 110, 8, 55, 17, 217, 73, 159, 223, 68, 135, 135, 59, 148, 166,
    222, 196, 231, 156, 96, 253, 17, 241, 19, 224, 159, 130, 60, 47, 225, 247,
    240, 87, 135, 109, 174, 19, 89, 240, 237, 188, 122, 197, 173, 197, 201,
    149, 238, 27, 79, 9, 251, 213, 184, 145, 155, 203, 126, 89, 134, 17, 114,
    24, 2, 0, 193, 175, 9, 248, 125, 225, 111, 136, 86, 158, 59, 143, 226, 15,
    194, 45, 64, 75, 60, 18, 173, 172, 208, 41, 62, 75, 220, 5, 18, 36, 5, 65,
    7, 107, 14, 65, 207, 94, 135, 156, 15, 3, 15, 149, 213, 193, 98, 161, 78,
    178, 82, 77, 116, 118, 252, 236, 244, 234, 236, 52, 239, 45, 116, 177,
    206, 252, 109, 248, 11, 241, 19, 225, 127, 137, 181, 75, 123, 171, 53,
    185, 210, 34, 188, 91, 125, 62, 234, 88, 214, 41, 111, 34, 194, 149, 109,
    153, 224, 224, 149, 61, 6, 83, 35, 174, 43, 153, 241, 119, 129, 103, 155,
    91, 130, 230, 195, 77, 145, 85, 236, 149, 183, 5, 192, 70, 249, 186, 254,
    149, 236, 63, 23, 62, 62, 248, 235, 226, 183, 138, 117, 111, 2, 248, 219,
    81, 183, 209, 111, 109, 138, 219, 195, 107, 12, 42, 177, 249, 232, 160,
    237, 103, 112, 100, 206, 125, 255, 0, 168, 166, 124, 46, 182, 241, 206,
    177, 225, 123, 203, 31, 22, 232, 179, 51, 8, 100, 251, 45, 210, 91, 179,
    41, 116, 24, 32, 176, 24, 239, 235, 94, 102, 115, 86, 80, 197, 63, 171,
    167, 238, 232, 211, 243, 252, 215, 153, 162, 105, 232, 217, 243, 118, 155,
    109, 168, 104, 183, 209, 221, 106, 154, 68, 179, 36, 51, 110, 144, 148,
    59, 91, 25, 245, 234, 43, 170, 127, 17, 248, 118, 87, 105, 23, 69, 218,
    28, 150, 3, 106, 240, 15, 227, 81, 106, 186, 55, 140, 91, 117, 245, 213,
    181, 203, 218, 41, 44, 226, 60, 16, 7, 81, 144, 57, 238, 63, 173, 114, 15,
    29, 225, 118, 41, 109, 49, 82, 78, 56, 35, 138, 186, 120, 135, 107, 232,
    95, 59, 167, 162, 71, 175, 252, 36, 248, 117, 23, 142, 110, 175, 111, 245,
    221, 91, 75, 131, 79, 179, 184, 210, 172, 94, 57, 228, 88, 111, 36, 138,
    226, 238, 24, 68, 214, 202, 227, 27, 17, 88, 163, 21, 99, 176, 180, 125,
    177, 159, 76, 248, 161, 251, 63, 77, 240, 239, 193, 86, 151, 250, 198,
    137, 246, 189, 19, 83, 179, 63, 216, 186, 234, 152, 24, 58, 205, 3, 77,
    246, 121, 192, 97, 34, 49, 198, 229, 35, 49, 185, 115, 128, 75, 40, 174,
    43, 225, 191, 196, 141, 91, 224, 103, 237, 16, 254, 19, 240, 214, 166,
    151, 94, 22, 131, 86, 26, 114, 63, 136, 165, 251, 39, 217, 44, 174, 30,
    48, 242, 121, 178, 48, 91, 96, 170, 204, 204, 220, 35, 162, 124, 195, 161,
    31, 80, 124, 70, 241, 238, 129, 227, 77, 99, 226, 46, 145, 225, 189, 111,
    67, 212, 252, 5, 160, 248, 106, 53, 54, 241, 95, 219, 205, 97, 121, 175,
    221, 195, 33, 134, 75, 73, 28, 200, 177, 165, 188, 44, 243, 74, 177, 21,
    95, 181, 48, 15, 181, 208, 77, 95, 75, 131, 142, 30, 120, 88, 202, 26,
    187, 62, 139, 170, 111, 206, 235, 95, 145, 206, 225, 107, 167, 184, 191,
    9, 53, 95, 136, 94, 41, 176, 240, 150, 161, 241, 147, 90, 125, 87, 76,
    241, 78, 153, 53, 200, 177, 147, 88, 91, 120, 227, 211, 225, 182, 30, 93,
    225, 89, 8, 89, 28, 128, 177, 186, 225, 84, 44, 141, 185, 195, 144, 107,
    133, 241, 63, 131, 252, 45, 225, 79, 7, 252, 93, 213, 254, 25, 252, 110,
    187, 212, 124, 41, 164, 91, 163, 201, 165, 65, 165, 189, 244, 241, 134,
    141, 2, 180, 23, 48, 76, 130, 53, 243, 131, 196, 102, 117, 97, 9, 140,
    151, 105, 79, 205, 94, 37, 224, 255, 0, 218, 3, 226, 135, 132, 237, 204,
    154, 110, 175, 12, 73, 225, 189, 30, 255, 0, 195, 22, 139, 61, 162, 200,
    35, 181, 188, 27, 198, 192, 197, 112, 194, 101, 80, 197, 155, 5, 66, 140,
    101, 74, 191, 216, 191, 0, 97, 248, 106, 158, 17, 240, 214, 157, 226, 77,
    31, 195, 26, 110, 179, 166, 218, 71, 164, 160, 179, 215, 173, 34, 150,
    123, 80, 136, 208, 164, 114, 207, 44, 114, 44, 178, 15, 32, 20, 36, 111,
    149, 55, 43, 72, 133, 86, 45, 176, 83, 134, 99, 21, 218, 27, 182, 157,
    245, 91, 93, 91, 201, 239, 170, 220, 119, 186, 185, 243, 175, 236, 205,
    241, 115, 84, 211, 60, 75, 226, 175, 12, 205, 225, 13, 10, 254, 67, 225,
    141, 79, 83, 182, 214, 103, 210, 252, 235, 187, 27, 183, 178, 49, 237,
    146, 108, 17, 36, 119, 18, 152, 213, 132, 216, 5, 192, 195, 28, 152, 100,
    245, 239, 139, 31, 23, 188, 45, 165, 120, 167, 192, 158, 44, 188, 210,
    117, 63, 13, 120, 78, 232, 141, 91, 73, 241, 110, 159, 108, 210, 75, 103,
    108, 97, 73, 161, 211, 132, 32, 180, 78, 161, 39, 65, 54, 228, 149, 200,
    86, 85, 137, 62, 250, 117, 223, 19, 126, 27, 104, 95, 11, 126, 29, 205,
    171, 124, 26, 159, 72, 22, 240, 222, 221, 232, 254, 39, 104, 164, 130,
    226, 125, 82, 1, 32, 46, 207, 28, 95, 187, 23, 81, 249, 155, 55, 72, 129,
    163, 66, 162, 63, 47, 228, 101, 225, 172, 190, 32, 120, 147, 74, 253, 157,
    252, 33, 225, 127, 8, 120, 114, 226, 13, 78, 230, 75, 155, 9, 117, 185,
    70, 251, 59, 107, 187, 89, 150, 215, 206, 93, 225, 151, 115, 71, 34, 194,
    97, 10, 165, 159, 45, 183, 106, 200, 225, 97, 231, 87, 13, 40, 81, 165,
    83, 155, 221, 190, 158, 77, 45, 21, 254, 87, 208, 19, 229, 247, 100, 181,
    71, 180, 191, 138, 190, 28, 124, 97, 240, 166, 166, 218, 110, 163, 225,
    63, 23, 226, 194, 86, 210, 237, 109, 35, 184, 210, 181, 24, 111, 23, 206,
    142, 127, 49, 38, 103, 104, 196, 204, 71, 150, 238, 155, 74, 200, 187,
    129, 24, 145, 190, 120, 248, 81, 30, 177, 240, 51, 196, 241, 248, 162, 63,
    14, 31, 21, 232, 158, 60, 16, 190, 146, 209, 197, 45, 180, 107, 34, 57,
    96, 232, 133, 73, 249, 6, 236, 185, 0, 97, 65, 202, 247, 224, 188, 89,
    227, 125, 79, 196, 62, 31, 240, 80, 248, 104, 182, 182, 122, 164, 250,
    121, 183, 212, 53, 75, 109, 62, 11, 9, 44, 46, 45, 148, 91, 11, 81, 44,
    103, 118, 197, 141, 12, 104, 50, 89, 145, 87, 146, 14, 23, 217, 254, 22,
    252, 92, 212, 254, 37, 105, 62, 9, 240, 63, 198, 175, 20, 88, 105, 22,
    186, 173, 221, 237, 196, 23, 45, 26, 193, 62, 187, 36, 141, 36, 150, 202,
    229, 0, 85, 89, 218, 72, 240, 170, 35, 89, 84, 194, 24, 22, 104, 152, 99,
    136, 204, 105, 84, 148, 83, 79, 154, 250, 63, 55, 109, 53, 236, 155, 186,
    123, 124, 139, 148, 90, 213, 30, 91, 241, 191, 192, 227, 198, 63, 29, 99,
    241, 87, 140, 252, 61, 171, 106, 30, 16, 181, 69, 184, 155, 74, 211, 154,
    43, 123, 216, 146, 77, 162, 73, 150, 104, 241, 230, 149, 37, 24, 130, 65,
    35, 30, 160, 31, 163, 147, 91, 208, 63, 103, 61, 54, 210, 24, 53, 29, 116,
    248, 86, 221, 99, 183, 187, 99, 122, 147, 53, 189, 195, 141, 240, 12, 156,
    182, 70, 198, 220, 49, 180, 239, 3, 60, 241, 242, 143, 197, 125, 127, 198,
    191, 14, 60, 99, 175, 120, 32, 107, 142, 154, 204, 22, 232, 47, 5, 204,
    106, 239, 119, 28, 203, 230, 184, 202, 145, 26, 96, 48, 4, 46, 57, 192,
    24, 10, 180, 219, 157, 31, 197, 145, 252, 26, 183, 215, 174, 126, 214,
    186, 31, 137, 109, 163, 50, 221, 92, 147, 60, 108, 209, 185, 117, 216,
    217, 38, 38, 12, 164, 21, 235, 128, 120, 202, 168, 60, 245, 243, 8, 194,
    83, 83, 131, 114, 87, 191, 91, 46, 151, 107, 166, 218, 141, 45, 57, 145,
    139, 251, 69, 235, 127, 8, 100, 214, 102, 241, 15, 193, 141, 70, 69, 143,
    87, 187, 55, 50, 90, 1, 50, 71, 106, 199, 230, 144, 4, 147, 59, 84, 185,
    97, 176, 112, 14, 236, 113, 138, 226, 173, 245, 77, 45, 224, 141, 231, 75,
    51, 43, 32, 46, 65, 31, 123, 28, 254, 181, 205, 216, 232, 118, 250, 204,
    26, 132, 207, 43, 70, 34, 150, 53, 14, 79, 10, 165, 176, 196, 143, 166,
    51, 93, 58, 248, 35, 193, 177, 1, 25, 241, 205, 190, 80, 109, 224, 140,
    113, 95, 53, 136, 157, 28, 68, 220, 154, 229, 242, 95, 211, 28, 106, 36,
    189, 228, 104, 120, 139, 68, 240, 236, 218, 254, 189, 175, 219, 199, 125,
    118, 250, 20, 51, 217, 106, 54, 128, 44, 114, 193, 59, 32, 132, 93, 198,
    28, 98, 72, 197, 195, 16, 241, 252, 173, 28, 143, 184, 54, 217, 3, 47,
    167, 255, 0, 195, 51, 252, 103, 240, 183, 132, 45, 180, 129, 175, 70, 60,
    11, 175, 219, 219, 107, 23, 215, 48, 68, 183, 54, 49, 125, 166, 40, 81,
    174, 21, 35, 111, 50, 227, 116, 45, 28, 112, 190, 232, 199, 239, 72, 220,
    140, 236, 79, 85, 240, 155, 192, 94, 36, 248, 239, 227, 115, 226, 63, 138,
    222, 15, 210, 188, 103, 225, 209, 167, 90, 233, 87, 218, 159, 159, 113,
    99, 169, 216, 139, 118, 145, 98, 185, 131, 12, 102, 149, 209, 218, 72, 75,
    48, 201, 138, 221, 23, 146, 190, 91, 125, 87, 169, 221, 124, 61, 208, 116,
    37, 241, 38, 159, 113, 172, 125, 143, 225, 214, 181, 169, 89, 189, 182,
    150, 162, 206, 11, 120, 132, 155, 47, 141, 234, 161, 43, 57, 66, 175, 187,
    4, 12, 220, 177, 149, 118, 59, 19, 244, 88, 92, 5, 227, 42, 148, 253, 232,
    183, 165, 157, 180, 79, 85, 217, 173, 246, 236, 101, 15, 121, 221, 187,
    31, 157, 186, 151, 134, 244, 175, 15, 252, 111, 214, 252, 49, 109, 226,
    75, 88, 180, 109, 51, 86, 46, 215, 154, 181, 161, 120, 166, 142, 57, 190,
    209, 230, 203, 110, 132, 153, 35, 223, 19, 110, 10, 195, 247, 123, 179,
    145, 149, 60, 255, 0, 133, 188, 89, 225, 47, 5, 120, 246, 223, 197, 46,
    250, 117, 197, 173, 182, 163, 30, 167, 120, 52, 221, 171, 5, 180, 176,
    220, 187, 197, 29, 188, 50, 163, 6, 10, 198, 57, 85, 36, 18, 33, 135, 112,
    113, 148, 108, 108, 124, 114, 248, 187, 115, 227, 47, 138, 26, 204, 182,
    183, 22, 23, 246, 16, 234, 218, 140, 182, 141, 110, 100, 184, 183, 157,
    82, 105, 19, 122, 151, 102, 97, 20, 209, 110, 99, 252, 59, 93, 73, 29, 90,
    190, 191, 248, 17, 99, 240, 91, 226, 175, 130, 252, 3, 167, 94, 91, 106,
    58, 183, 136, 252, 42, 126, 203, 107, 172, 106, 74, 182, 186, 181, 148,
    43, 20, 160, 180, 51, 8, 26, 41, 146, 34, 27, 202, 71, 50, 21, 242, 178,
    75, 188, 106, 66, 192, 97, 175, 94, 113, 163, 45, 34, 238, 190, 251, 61,
    47, 125, 63, 34, 105, 223, 225, 56, 63, 137, 223, 181, 231, 136, 173, 124,
    20, 159, 15, 188, 21, 224, 175, 176, 120, 163, 93, 191, 154, 247, 81, 131,
    84, 179, 51, 44, 250, 117, 216, 221, 109, 118, 173, 3, 164, 145, 92, 35,
    20, 129, 214, 66, 87, 113, 38, 48, 202, 118, 71, 179, 240, 131, 246, 190,
    209, 135, 129, 99, 180, 241, 118, 171, 167, 248, 126, 235, 70, 54, 22,
    179, 88, 89, 233, 65, 173, 245, 37, 120, 165, 142, 226, 243, 80, 136, 33,
    204, 177, 76, 98, 144, 199, 167, 180, 55, 19, 202, 228, 186, 108, 59, 224,
    209, 248, 69, 31, 198, 251, 247, 248, 235, 240, 218, 247, 225, 247, 133,
    53, 159, 18, 220, 106, 15, 52, 150, 118, 44, 91, 77, 146, 123, 189, 56,
    37, 137, 51, 25, 85, 228, 182, 149, 32, 40, 159, 48, 101, 242, 229, 50,
    254, 240, 146, 190, 53, 225, 191, 217, 231, 197, 183, 130, 111, 5, 232,
    176, 105, 122, 214, 165, 118, 145, 73, 127, 107, 127, 102, 34, 107, 75,
    61, 204, 178, 94, 136, 228, 30, 100, 139, 28, 138, 177, 174, 192, 173,
    190, 100, 119, 88, 149, 177, 93, 21, 225, 139, 167, 40, 253, 86, 242, 91,
    91, 107, 107, 253, 109, 249, 26, 55, 173, 203, 190, 63, 147, 194, 94, 32,
    159, 92, 240, 45, 182, 177, 117, 174, 105, 183, 159, 101, 107, 143, 20,
    221, 219, 201, 13, 210, 222, 134, 80, 176, 165, 185, 40, 99, 145, 109, 98,
    134, 9, 139, 135, 145, 118, 207, 8, 39, 49, 172, 30, 123, 161, 248, 255,
    0, 79, 208, 190, 37, 90, 93, 120, 243, 80, 187, 241, 70, 164, 179, 67,
    105, 44, 170, 208, 37, 138, 105, 236, 136, 178, 67, 36, 49, 41, 140, 108,
    141, 66, 24, 208, 43, 43, 171, 46, 85, 148, 20, 232, 190, 18, 233, 22, 19,
    248, 139, 195, 186, 166, 189, 125, 119, 241, 23, 91, 214, 36, 109, 23, 74,
    209, 44, 238, 37, 180, 183, 150, 242, 101, 142, 210, 7, 184, 213, 46, 21,
    69, 172, 66, 8, 96, 43, 176, 75, 35, 131, 130, 16, 57, 97, 235, 127, 15,
    191, 102, 15, 1, 107, 218, 70, 159, 227, 13, 103, 199, 17, 248, 118, 218,
    107, 251, 171, 125, 98, 222, 61, 122, 217, 172, 109, 90, 123, 185, 160,
    183, 242, 222, 69, 75, 136, 151, 207, 107, 114, 171, 49, 222, 235, 33, 71,
    217, 42, 24, 95, 154, 88, 106, 217, 142, 33, 212, 130, 73, 71, 163, 118,
    221, 245, 122, 233, 247, 187, 92, 109, 205, 73, 52, 112, 159, 180, 76,
    214, 223, 26, 52, 75, 127, 29, 248, 30, 116, 213, 45, 52, 61, 58, 219, 81,
    187, 138, 86, 138, 25, 237, 30, 103, 154, 39, 181, 68, 0, 79, 42, 44, 80,
    164, 142, 143, 38, 232, 252, 232, 136, 47, 146, 170, 159, 6, 255, 0, 105,
    239, 134, 243, 233, 147, 252, 33, 248, 137, 225, 139, 88, 188, 63, 172,
    162, 91, 76, 237, 113, 4, 86, 176, 202, 84, 1, 40, 138, 11, 120, 140, 101,
    157, 228, 98, 230, 70, 35, 229, 11, 183, 21, 133, 240, 87, 192, 122, 205,
    255, 0, 196, 171, 157, 77, 245, 75, 91, 189, 23, 78, 188, 212, 52, 213,
    143, 86, 189, 123, 8, 181, 59, 116, 156, 194, 84, 79, 131, 36, 140, 64,
    36, 171, 35, 44, 96, 176, 40, 119, 21, 175, 160, 254, 42, 124, 25, 240,
    54, 161, 226, 79, 22, 120, 151, 194, 22, 218, 101, 150, 143, 165, 104, 16,
    106, 87, 26, 180, 214, 144, 182, 159, 114, 38, 51, 171, 181, 172, 251,
    133, 191, 150, 190, 83, 44, 159, 189, 1, 29, 25, 73, 219, 144, 106, 143,
    182, 197, 77, 212, 131, 73, 252, 45, 61, 83, 183, 69, 117, 183, 77, 203,
    167, 52, 157, 228, 174, 124, 105, 162, 217, 104, 62, 31, 241, 198, 163,
    225, 157, 75, 78, 185, 188, 240, 164, 183, 147, 192, 147, 153, 149, 132,
    160, 51, 24, 208, 72, 188, 54, 85, 125, 137, 235, 197, 116, 210, 248, 171,
    193, 145, 74, 241, 218, 124, 57, 240, 66, 192, 140, 86, 37, 151, 81, 136,
    56, 64, 120, 12, 51, 215, 24, 205, 114, 119, 83, 104, 118, 218, 101, 173,
    135, 135, 181, 24, 111, 210, 61, 106, 98, 247, 162, 221, 98, 70, 144, 163,
    50, 196, 2, 73, 34, 50, 140, 42, 239, 86, 32, 239, 63, 221, 57, 230, 87,
    224, 172, 90, 138, 141, 65, 36, 213, 81, 110, 135, 156, 21, 108, 114, 0,
    110, 112, 14, 121, 28, 215, 203, 213, 163, 5, 94, 113, 196, 62, 70, 159,
    71, 117, 191, 147, 53, 229, 228, 118, 134, 214, 79, 107, 238, 119, 90,
    127, 237, 67, 226, 207, 14, 253, 162, 251, 194, 122, 128, 211, 117, 59,
    57, 163, 179, 91, 120, 150, 72, 161, 212, 52, 255, 0, 58, 57, 54, 249,
    138, 222, 110, 76, 176, 167, 152, 133, 215, 122, 72, 251, 183, 124, 203,
    95, 91, 120, 235, 226, 182, 187, 251, 65, 252, 57, 241, 247, 134, 36, 54,
    62, 30, 211, 167, 210, 180, 245, 134, 59, 109, 92, 189, 226, 52, 134, 50,
    200, 98, 36, 51, 196, 179, 71, 42, 23, 9, 26, 108, 148, 169, 70, 18, 185,
    135, 243, 87, 197, 150, 208, 233, 30, 40, 213, 236, 244, 224, 241, 69,
    105, 171, 92, 193, 15, 206, 204, 202, 145, 202, 225, 6, 226, 73, 56, 0,
    114, 78, 120, 175, 71, 241, 87, 142, 60, 75, 163, 124, 40, 248, 105, 226,
    205, 62, 246, 20, 213, 167, 93, 71, 125, 211, 217, 195, 35, 22, 142, 242,
    225, 81, 240, 232, 64, 117, 18, 48, 14, 6, 225, 198, 15, 202, 184, 250,
    220, 28, 30, 30, 148, 169, 211, 217, 107, 110, 143, 163, 91, 55, 103, 115,
    153, 70, 215, 71, 14, 247, 239, 167, 178, 136, 108, 160, 134, 226, 19, 34,
    59, 164, 140, 88, 31, 153, 74, 25, 20, 237, 96, 70, 255, 0, 187, 242, 178,
    183, 25, 25, 7, 220, 180, 159, 138, 95, 25, 252, 35, 225, 143, 12, 106,
    179, 105, 215, 118, 154, 53, 149, 196, 112, 104, 83, 203, 160, 188, 22,
    242, 219, 178, 207, 137, 140, 177, 42, 52, 219, 213, 174, 66, 230, 66, 89,
    188, 210, 138, 91, 204, 106, 241, 237, 42, 202, 219, 86, 240, 55, 137,
    124, 65, 169, 198, 110, 181, 47, 237, 11, 52, 55, 82, 177, 105, 8, 154,
    87, 50, 252, 196, 231, 44, 70, 73, 235, 201, 245, 53, 250, 241, 251, 46,
    120, 51, 194, 159, 19, 127, 100, 175, 135, 58, 39, 142, 244, 11, 61, 95,
    79, 248, 133, 21, 254, 153, 226, 59, 105, 99, 11, 29, 244, 54, 82, 92,
    195, 104, 219, 83, 2, 41, 162, 142, 222, 221, 82, 120, 246, 202, 60, 136,
    142, 252, 198, 164, 112, 80, 157, 69, 55, 26, 78, 221, 31, 224, 215, 174,
    182, 42, 20, 212, 147, 103, 139, 124, 51, 248, 187, 240, 63, 196, 191,
    179, 222, 141, 31, 143, 60, 79, 6, 144, 218, 149, 238, 255, 0, 20, 233,
    186, 78, 157, 115, 21, 198, 155, 34, 106, 41, 155, 217, 238, 109, 34, 79,
    46, 60, 203, 4, 210, 203, 136, 247, 60, 251, 81, 188, 194, 174, 155, 90,
    191, 141, 62, 53, 107, 62, 63, 241, 62, 183, 240, 231, 196, 90, 52, 122,
    77, 150, 155, 109, 226, 27, 13, 82, 71, 123, 141, 63, 196, 23, 130, 38,
    104, 173, 196, 247, 18, 199, 111, 28, 47, 20, 82, 44, 215, 94, 99, 21,
    243, 144, 12, 111, 98, 60, 7, 246, 143, 240, 174, 141, 240, 67, 94, 241,
    133, 159, 194, 168, 238, 252, 59, 13, 183, 194, 253, 42, 40, 214, 218,
    250, 118, 249, 53, 13, 83, 236, 151, 170, 219, 221, 183, 137, 97, 102, 82,
    27, 56, 99, 188, 97, 192, 106, 250, 79, 224, 53, 133, 190, 141, 224, 45,
    37, 52, 255, 0, 53, 33, 183, 211, 124, 111, 97, 21, 188, 147, 60, 176, 44,
    26, 110, 191, 103, 5, 146, 249, 78, 74, 102, 56, 222, 65, 187, 27, 152,
    187, 179, 150, 102, 36, 251, 56, 58, 142, 165, 71, 42, 155, 171, 39, 110,
    186, 255, 0, 193, 185, 74, 45, 251, 183, 216, 249, 203, 69, 214, 254, 35,
    124, 36, 241, 183, 139, 126, 40, 124, 61, 248, 113, 105, 55, 135, 100,
    176, 213, 175, 244, 109, 42, 235, 81, 138, 88, 244, 221, 8, 36, 145, 163,
    195, 229, 177, 119, 142, 25, 100, 89, 102, 182, 143, 39, 109, 187, 69, 33,
    30, 90, 200, 188, 183, 236, 231, 241, 195, 196, 182, 158, 57, 79, 22, 252,
    75, 142, 77, 107, 69, 213, 100, 107, 75, 97, 45, 157, 180, 112, 199, 42,
    92, 91, 76, 211, 196, 27, 8, 178, 169, 181, 133, 154, 115, 202, 60, 48,
    177, 147, 120, 221, 95, 161, 186, 191, 134, 180, 43, 79, 140, 86, 218,
    173, 190, 155, 18, 92, 233, 222, 13, 212, 45, 109, 155, 146, 171, 20, 119,
    90, 100, 136, 133, 79, 202, 193, 93, 152, 141, 192, 227, 115, 14, 132,
    138, 248, 3, 254, 10, 15, 105, 107, 99, 251, 84, 106, 144, 88, 192, 150,
    201, 168, 89, 105, 51, 92, 136, 70, 207, 50, 71, 15, 11, 177, 199, 115,
    28, 49, 169, 61, 78, 223, 82, 73, 188, 109, 57, 101, 235, 218, 210, 119,
    138, 150, 207, 211, 202, 221, 204, 185, 164, 157, 174, 86, 241, 47, 196,
    45, 47, 91, 240, 221, 247, 137, 165, 188, 54, 222, 34, 213, 36, 125, 67,
    82, 152, 238, 158, 223, 77, 105, 84, 236, 88, 213, 219, 106, 221, 74, 192,
    5, 110, 145, 168, 223, 33, 221, 177, 89, 62, 29, 124, 126, 248, 161, 163,
    248, 7, 195, 126, 31, 240, 212, 94, 26, 183, 240, 127, 132, 111, 174, 190,
    221, 29, 252, 182, 182, 246, 186, 164, 87, 27, 217, 172, 36, 134, 119,
    223, 117, 14, 201, 110, 1, 17, 36, 178, 55, 218, 48, 225, 217, 147, 119,
    133, 248, 143, 82, 212, 52, 45, 90, 218, 29, 30, 250, 226, 206, 56, 212,
    0, 176, 200, 84, 115, 201, 39, 212, 156, 156, 147, 201, 239, 87, 238, 47,
    38, 215, 116, 61, 107, 82, 212, 132, 111, 117, 97, 48, 72, 37, 142, 37,
    132, 128, 102, 42, 119, 108, 3, 127, 3, 248, 179, 220, 245, 38, 190, 82,
    56, 186, 145, 147, 196, 197, 217, 207, 123, 105, 190, 191, 153, 151, 51,
    86, 72, 181, 226, 27, 159, 9, 61, 237, 205, 143, 134, 116, 211, 165, 232,
    230, 241, 238, 237, 172, 45, 231, 51, 45, 185, 110, 6, 231, 32, 9, 37, 85,
    194, 177, 140, 121, 101, 149, 202, 110, 82, 9, 245, 27, 31, 218, 63, 196,
    150, 86, 86, 246, 118, 255, 0, 13, 180, 198, 138, 8, 146, 36, 59, 227, 92,
    170, 128, 7, 27, 184, 224, 116, 174, 55, 226, 47, 134, 116, 29, 31, 65,
    180, 155, 76, 211, 34, 183, 112, 150, 232, 89, 51, 150, 4, 57, 203, 115,
    243, 28, 168, 228, 228, 245, 231, 147, 94, 205, 225, 143, 2, 120, 90, 243,
    195, 122, 77, 221, 206, 154, 239, 52, 246, 48, 73, 35, 125, 166, 81, 185,
    154, 53, 36, 224, 54, 58, 154, 240, 49, 82, 133, 71, 205, 53, 125, 94,
    247, 253, 25, 221, 74, 181, 88, 212, 146, 82, 63, 255, 217
};

const unsigned char hana_bmp[RGB_NUM][BMP_OUT_SIZE] = {
  {188, 209, 190, 153, 143, 103, 149, 192, 172, 157, 189, 192, 164, 195, 188,
   181, 193, 167, 188, 210, 177, 142, 37, 10, 31, 40, 39, 52, 32, 29, 34, 14,
   10, 18, 14, 26, 7, 108, 181, 190, 181, 127, 91, 99, 73, 36, 11, 36, 66, 93,
   90, 65, 51, 32, 36, 35, 38, 53, 56, 35, 25, 25, 32, 49, 41, 54, 32, 31, 31,
   34, 24, 25, 38, 43, 32, 11, 9, 13, 20, 25, 23, 17, 16, 18, 30, 25, 33, 45,
   36, 29, 166, 193, 101, 90, 170, 129, 146, 213, 169, 88, 145, 199, 207, 165,
   137, 113, 201, 155, 139, 209, 191, 201, 171, 103, 51, 29, 17, 9, 29, 43,
   33, 36, 24, 14, 24, 24, 4, 108, 185, 180, 186, 188, 183, 195, 192, 108, 48,
   75, 103, 85, 69, 49, 46, 55, 44, 42, 50, 51, 41, 30, 18, 14, 30, 48, 48,
   54, 34, 40, 38, 36, 31, 28, 52, 55, 45, 15, 14, 16, 23, 21, 15, 11, 17, 25,
   31, 26, 44, 73, 34, 39, 145, 132, 46, 181, 205, 209, 187, 170, 140, 56, 52,
   142, 122, 176, 198, 83, 154, 215, 200, 209, 184, 164, 196, 174, 128, 55,
   89, 76, 28, 38, 47, 46, 35, 27, 39, 31, 16, 58, 116, 137, 169, 184, 180,
   175, 166, 120, 73, 91, 97, 84, 32, 36, 62, 58, 40, 41, 48, 45, 32, 20, 15,
   16, 12, 27, 24, 36, 40, 16, 34, 24, 22, 33, 61, 65, 55, 18, 21, 22, 29, 19,
   18, 15, 21, 28, 33, 29, 48, 75, 36, 38, 111, 15, 62, 226, 207, 187, 205,
   188, 61, 2, 13, 31, 9, 105, 140, 49, 24, 191, 209, 198, 186, 184, 147, 117,
   89, 58, 138, 179, 135, 62, 52, 51, 23, 20, 34, 28, 26, 10, 42, 112, 151,
   157, 139, 122, 113, 54, 79, 84, 96, 46, 19, 32, 53, 58, 40, 40, 39, 38, 21,
   21, 17, 23, 6, 110, 101, 14, 18, 7, 15, 27, 7, 27, 64, 70, 63, 22, 30, 32,
   38, 22, 8, 10, 18, 20, 26, 27, 43, 58, 44, 40, 139, 140, 129, 156, 113,
   177, 215, 174, 111, 101, 36, 14, 20, 139, 98, 106, 81, 37, 97, 223, 200,
   217, 149, 153, 148, 131, 70, 146, 178, 172, 131, 80, 13, 17, 43, 16, 18,
   20, 24, 98, 130, 163, 144, 85, 71, 81, 78, 88, 68, 29, 39, 56, 59, 49, 40,
   41, 39, 20, 11, 63, 80, 1, 89, 179, 186, 96, 0, 37, 15, 21, 20, 27, 64, 76,
   73, 29, 31, 33, 43, 34, 13, 12, 19, 16, 16, 14, 28, 38, 60, 50, 179, 196,
   217, 132, 55, 121, 213, 180, 177, 183, 174, 102, 161, 182, 140, 147, 197,
   206, 164, 214, 217, 175, 40, 141, 174, 183, 123, 68, 97, 119, 123, 126, 49,
   21, 39, 17, 23, 28, 14, 123, 149, 194, 193, 95, 75, 73, 70, 63, 59, 50, 48,
   55, 58, 42, 48, 54, 32, 36, 12, 85, 159, 110, 125, 177, 187, 137, 25, 54,
   58, 55, 18, 19, 59, 77, 76, 34, 30, 32, 48, 47, 34, 18, 13, 13, 12, 5, 12,
   18, 52, 57, 136, 134, 175, 214, 134, 161, 190, 141, 177, 189, 141, 164,
   203, 179, 185, 144, 73, 141, 206, 148, 181, 192, 113, 69, 149, 176, 141,
   116, 83, 46, 101, 112, 79, 31, 25, 40, 30, 26, 12, 98, 162, 195, 215, 122,
   55, 65, 61, 68, 64, 46, 68, 65, 67, 80, 56, 36, 44, 47, 27, 55, 177, 200,
   175, 169, 178, 156, 114, 126, 107, 61, 46, 50, 60, 78, 76, 46, 42, 48, 59,
   58, 45, 25, 20, 22, 26, 19, 19, 15, 56, 58, 41, 114, 196, 195, 208, 197,
   186, 176, 174, 204, 121, 136, 208, 205, 201, 86, 12, 190, 180, 177, 148,
   178, 221, 145, 97, 146, 136, 153, 169, 138, 147, 191, 167, 57, 47, 168,
   124, 33, 5, 54, 189, 180, 197, 188, 136, 68, 54, 63, 50, 78, 83, 74, 80,
   90, 58, 44, 60, 45, 44, 18, 128, 194, 176, 201, 203, 196, 175, 142, 107,
   61, 44, 61, 74, 86, 83, 62, 64, 71, 72, 63, 63, 55, 57, 52, 49, 39, 36, 24,
   61, 57, 19, 75, 194, 165, 168, 186, 196, 187, 203, 207, 86, 65, 130, 192,
   214, 184, 144, 174, 180, 209, 147, 76, 108, 116, 155, 154, 187, 172, 179,
   180, 187, 199, 204, 57, 54, 165, 127, 47, 90, 67, 189, 179, 170, 214, 195,
   159, 103, 40, 32, 64, 97, 66, 53, 53, 67, 77, 46, 47, 39, 10, 53, 122, 151,
   202, 213, 214, 188, 161, 145, 106, 60, 58, 78, 93, 97, 101, 102, 86, 85,
   71, 57, 50, 55, 54, 54, 52, 43, 40, 21, 22, 113, 11, 46, 66, 104, 140, 164,
   167, 177, 210, 71, 115, 105, 190, 163, 181, 187, 184, 142, 177, 107, 207,
   128, 113, 121, 156, 153, 143, 161, 154, 165, 182, 182, 55, 45, 116, 42, 52,
   120, 94, 125, 180, 187, 186, 199, 191, 186, 78, 0, 43, 96, 118, 82, 19, 76,
   76, 71, 55, 29, 9, 8, 59, 111, 168, 189, 205, 187, 166, 138, 111, 50, 41,
   55, 76, 94, 107, 116, 101, 92, 72, 51, 50, 47, 39, 43, 37, 29, 40, 25, 27,
   100, 68, 51, 46, 56, 81, 92, 152, 176, 202, 71, 142, 207, 196, 197, 144,
   181, 181, 105, 120, 79, 64, 179, 112, 102, 97, 64, 63, 51, 94, 180, 164,
   147, 37, 68, 169, 94, 100, 97, 138, 1, 68, 178, 181, 173, 166, 182, 163,
   51, 119, 165, 187, 187, 65, 39, 79, 82, 71, 15, 14, 10, 12, 64, 144, 185,
   176, 116, 70, 60, 41, 39, 25, 23, 43, 63, 86, 110, 110, 100, 75, 49, 27,
   32, 27, 18, 21, 36, 37, 29, 32, 93, 62, 34, 31, 71, 62, 102, 175, 144, 106,
   58, 201, 170, 191, 189, 167, 110, 170, 174, 134, 121, 84, 161, 120, 85, 62,
   64, 43, 9, 78, 186, 185, 170, 20, 100, 180, 159, 176, 172, 189, 100, 106,
   169, 170, 153, 180, 189, 187, 191, 150, 103, 138, 155, 125, 49, 77, 61, 62,
   72, 23, 18, 21, 27, 39, 82, 80, 53, 53, 71, 84, 58, 33, 4, 18, 35, 63, 101,
   120, 107, 70, 39, 28, 29, 25, 28, 24, 32, 33, 34, 31, 98, 59, 32, 17, 34,
   28, 69, 121, 26, 46, 76, 172, 107, 19, 142, 153, 136, 146, 214, 165, 187,
   207, 179, 126, 63, 81, 38, 45, 38, 97, 173, 184, 155, 24, 100, 206, 176,
   198, 206, 226, 152, 107, 129, 153, 146, 188, 187, 170, 179, 179, 124, 155,
   179, 154, 93, 73, 65, 81, 66, 57, 20, 58, 60, 36, 47, 56, 25, 49, 74, 90,
   85, 44, 8, 12, 24, 48, 83, 122, 120, 87, 37, 29, 28, 23, 23, 20, 29, 31,
   23, 20, 97, 51, 9, 15, 14, 16, 23, 88, 32, 37, 75, 48, 18, 17, 24, 197,
   200, 170, 196, 191, 202, 205, 182, 100, 56, 35, 28, 113, 92, 71, 93, 86,
   86, 34, 78, 186, 194, 182, 193, 180, 165, 78, 86, 122, 64, 105, 195, 163,
   148, 188, 148, 150, 150, 118, 120, 95, 73, 77, 75, 70, 63, 79, 95, 64, 82,
   59, 31, 20, 43, 70, 93, 84, 29, 29, 44, 62, 78, 112, 120, 93, 44, 34, 34,
   35, 17, 17, 28, 20, 23, 26, 85, 58, 21, 24, 29, 17, 13, 31, 13, 21, 89, 31,
   45, 19, 21, 146, 225, 210, 206, 185, 66, 164, 183, 191, 149, 60, 66, 119,
   52, 74, 55, 142, 158, 20, 75, 154, 161, 201, 147, 138, 182, 143, 122, 84,
   21, 116, 174, 174, 168, 198, 128, 73, 78, 166, 159, 78, 73, 53, 36, 113,
   124, 88, 128, 127, 106, 66, 9, 23, 20, 41, 74, 89, 65, 52, 62, 73, 63, 91,
   111, 98, 54, 70, 61, 63, 60, 50, 29, 22, 25, 28, 37, 80, 25, 23, 16, 4, 11,
   22, 13, 22, 86, 29, 18, 50, 25, 8, 119, 163, 160, 185, 161, 158, 189, 179,
   132, 119, 116, 39, 129, 171, 137, 180, 174, 19, 117, 194, 192, 193, 174,
   159, 119, 165, 177, 168, 44, 121, 161, 185, 200, 194, 167, 82, 82, 162,
   162, 74, 51, 55, 15, 99, 156, 118, 127, 177, 163, 72, 9, 13, 10, 5, 37, 71,
   77, 70, 109, 141, 123, 133, 137, 117, 73, 79, 69, 83, 80, 82, 67, 47, 28,
   26, 13, 77, 23, 18, 15, 16, 16, 34, 33, 21, 85, 14, 0, 37, 44, 0, 179, 225,
   187, 206, 209, 141, 149, 78, 57, 63, 72, 129, 159, 164, 109, 97, 115, 31,
   36, 91, 25, 93, 215, 108, 100, 180, 181, 120, 68, 82, 168, 195, 194, 149,
   185, 179, 131, 72, 131, 75, 55, 46, 32, 61, 144, 150, 129, 196, 182, 121,
   87, 81, 6, 5, 20, 38, 56, 100, 153, 172, 204, 204, 198, 212, 86, 32, 65,
   51, 59, 57, 59, 52, 51, 44, 43, 30, 32, 20, 50, 70, 49, 19, 26, 40, 69,
   100, 75, 50, 40, 18, 100, 192, 104, 100, 105, 118, 131, 129, 42, 76, 35,
   73, 124, 146, 104, 1, 28, 75, 98, 197, 166, 177, 212, 201, 175, 174, 192,
   142, 99, 33, 87, 216, 189, 193, 209, 185, 188, 126, 126, 92, 86, 58, 35,
   28, 116, 174, 181, 188, 180, 166, 136, 86, 16, 1, 0, 6, 23, 63, 103, 136,
   174, 192, 155, 198, 173, 35, 16, 42, 51, 40, 38, 58, 43, 44, 26, 29, 25,
   14, 37, 75, 35, 59, 19, 59, 61, 101, 55, 81, 79, 29, 48, 86, 109, 81, 66,
   187, 143, 182, 142, 13, 2, 39, 78, 144, 105, 28, 34, 84, 27, 93, 253, 169,
   242, 93, 216, 161, 155, 194, 231, 139, 7, 91, 205, 204, 226, 184, 223, 234,
   190, 112, 92, 43, 37, 20, 132, 194, 198, 203, 174, 138, 58, 7, 0, 26, 102,
   110, 91, 121, 120, 132, 109, 171, 167, 165, 217, 176, 82, 77, 68, 51, 22,
   38, 44, 44, 5, 22, 29, 29, 7, 99, 30, 139, 101, 53, 60, 100, 135, 76, 64,
   32, 36, 51, 54, 87, 111, 147, 240, 186, 172, 205, 46, 0, 46, 21, 11, 62,
   37, 45, 26, 0, 46, 194, 186, 60, 239, 98, 140, 192, 121, 19, 178, 164, 94,
   183, 67, 87, 194, 194, 196, 190, 83, 34, 32, 56, 85, 171, 182, 175, 206,
   153, 61, 1, 6, 64, 161, 198, 179, 172, 179, 170, 112, 162, 221, 191, 185,
   181, 136, 90, 86, 57, 29, 27, 44, 43, 53, 20, 12, 18, 33, 96, 56, 156, 139,
   132, 61, 47, 117, 113, 101, 36, 27, 5, 63, 166, 123, 21, 163, 207, 224,
   180, 129, 212, 159, 193, 205, 42, 20, 51, 23, 27, 17, 48, 240, 52, 177,
   177, 149, 64, 1, 140, 186, 180, 204, 211, 213, 101, 187, 198, 150, 58, 112,
   129, 86, 55, 63, 77, 74, 102, 146, 154, 99, 53, 48, 28, 87, 179, 177, 175,
   178, 175, 116, 134, 157, 178, 193, 180, 148, 125, 85, 28, 20, 18, 58, 59,
   31, 43, 10, 71, 177, 146, 45, 153, 118, 184, 141, 51, 27, 56, 77, 45, 30,
   72, 145, 159, 90, 88, 168, 142, 183, 212, 86, 173, 204, 201, 132, 0, 33,
   49, 21, 9, 0, 125, 190, 61, 81, 177, 95, 72, 23, 15, 96, 173, 214, 180,
   229, 218, 186, 185, 223, 72, 146, 160, 129, 86, 62, 78, 66, 93, 111, 113,
   90, 73, 82, 74, 3, 66, 150, 191, 181, 104, 120, 90, 116, 114, 125, 171,
   122, 132, 66, 21, 25, 27, 59, 60, 37, 33, 13, 26, 201, 191, 52, 170, 137,
   189, 217, 208, 152, 26, 111, 40, 10, 148, 169, 162, 153, 134, 204, 203,
   171, 146, 61, 101, 202, 142, 0, 9, 44, 54, 19, 12, 0, 36, 167, 82, 28, 217,
   123, 102, 12, 53, 23, 0, 47, 0, 195, 194, 188, 172, 176, 174, 61, 101, 157,
   136, 109, 40, 44, 125, 146, 147, 128, 77, 97, 113, 83, 54, 172, 190, 165,
   159, 191, 85, 65, 63, 33, 45, 63, 60, 33, 21, 19, 16, 63, 61, 34, 51, 57,
   48, 36, 137, 24, 122, 90, 230, 194, 210, 190, 80, 42, 51, 18, 149, 211,
   159, 39, 123, 172, 96, 63, 9, 74, 42, 148, 224, 119, 0, 53, 52, 30, 0, 43,
   189, 117, 94, 37, 159, 157, 214, 103, 22, 30, 13, 58, 60, 44, 67, 100, 168,
   163, 179, 54, 139, 219, 193, 113, 68, 140, 143, 141, 165, 121, 126, 87,
   146, 124, 95, 166, 167, 130, 156, 189, 108, 42, 43, 41, 17, 40, 30, 29, 23,
   19, 18, 61, 60, 13, 35, 94, 122, 50, 117, 205, 172, 147, 108, 221, 186,
   183, 137, 38, 37, 31, 106, 208, 156, 18, 175, 229, 139, 0, 105, 114, 99,
   12, 146, 141, 162, 52, 42, 0, 62, 177, 186, 136, 94, 32, 123, 146, 210,
   193, 108, 168, 145, 8, 193, 36, 61, 69, 61, 166, 156, 203, 127, 195, 228,
   146, 169, 141, 90, 136, 109, 95, 98, 90, 153, 143, 86, 82, 97, 96, 99, 127,
   132, 38, 46, 36, 30, 24, 35, 23, 25, 27, 12, 16, 30, 14, 0, 4, 22, 114,
   153, 158, 173, 188, 188, 47, 112, 248, 109, 205, 46, 34, 79, 181, 106, 44,
   182, 201, 141, 0, 150, 172, 126, 170, 160, 134, 122, 107, 55, 204, 109, 62,
   152, 235, 92, 38, 134, 101, 183, 217, 148, 152, 159, 115, 191, 176, 155,
   144, 95, 62, 52, 186, 146, 216, 178, 251, 96, 73, 126, 138, 56, 97, 55,
   103, 120, 126, 73, 48, 34, 57, 48, 141, 138, 57, 33, 34, 35, 23, 22, 31,
   28, 20, 23, 33, 30, 46, 1, 19, 29, 27, 124, 164, 172, 202, 218, 176, 100,
   24, 110, 189, 50, 44, 37, 99, 29, 31, 98, 164, 48, 71, 197, 153, 202, 167,
   156, 196, 192, 56, 128, 217, 175, 125, 153, 49, 106, 15, 128, 239, 182,
   195, 171, 92, 37, 110, 86, 118, 148, 98, 55, 34, 30, 21, 48, 232, 209, 106,
   87, 107, 172, 30, 71, 86, 88, 72, 62, 49, 66, 46, 64, 80, 68, 76, 111, 45,
   42, 35, 34, 28, 26, 49, 43, 22, 29, 32, 32, 144, 41, 0, 21, 2, 0, 88, 160,
   167, 192, 168, 87, 0, 169, 235, 50, 35, 22, 12, 98, 0, 23, 61, 0, 71, 53,
   240, 144, 206, 178, 188, 225, 200, 167, 206, 223, 118, 12, 75, 109, 36, 81,
   225, 194, 144, 182, 73, 32, 32, 112, 167, 93, 50, 44, 30, 17, 51, 43, 93,
   117, 113, 73, 210, 162, 99, 87, 87, 65, 34, 34, 22, 21, 35, 72, 103, 99,
   58, 56, 47, 45, 40, 26, 30, 34, 62, 57, 32, 37, 30, 30, 183, 112, 0, 8, 11,
   26, 0, 119, 205, 148, 210, 0, 60, 225, 83, 58, 35, 30, 6, 37, 61, 7, 105,
   31, 36, 7, 89, 229, 171, 172, 195, 219, 211, 175, 177, 219, 144, 74, 47,
   106, 37, 50, 194, 188, 100, 83, 25, 64, 49, 187, 175, 132, 25, 43, 25, 32,
   48, 30, 74, 108, 90, 139, 210, 197, 131, 106, 79, 42, 86, 64, 35, 36, 38,
   68, 93, 87, 103, 54, 87, 41, 41, 21, 24, 29, 61, 58, 38, 55, 20, 30, 145,
   76, 5, 10, 2, 35, 0, 145, 191, 215, 59, 5, 150, 179, 137, 46, 44, 24, 9,
   24, 70, 62, 21, 11, 65, 141, 87, 127, 173, 139, 169, 108, 212, 181, 150,
   194, 202, 154, 82, 58, 80, 60, 89, 141, 84, 0, 104, 104, 26, 82, 161, 85,
   12, 16, 60, 76, 72, 2, 82, 124, 97, 162, 241, 185, 116, 145, 42, 49, 88,
   68, 42, 20, 27, 65, 85, 100, 79, 93, 58, 76, 30, 22, 21, 28, 61, 58, 37,
   65, 32, 28, 99, 40, 5, 9, 11, 23, 42, 220, 196, 255, 0, 26, 226, 164, 130,
   110, 29, 31, 0, 31, 70, 41, 37, 39, 102, 126, 156, 209, 158, 215, 249, 226,
   222, 195, 232, 226, 202, 163, 93, 63, 84, 51, 131, 157, 0, 139, 75, 114,
   59, 11, 124, 94, 28, 69, 110, 97, 47, 44, 145, 104, 99, 39, 181, 188, 185,
   183, 54, 18, 39, 46, 67, 45, 7, 103, 163, 134, 109, 77, 75, 40, 26, 22, 24,
   35, 62, 64, 37, 50, 32, 31, 135, 104, 10, 2, 20, 20, 26, 0, 30, 14, 21, 79,
   136, 108, 101, 38, 56, 22, 25, 31, 65, 61, 14, 22, 59, 119, 58, 82, 98, 74,
   41, 157, 200, 72, 82, 218, 157, 166, 121, 82, 113, 17, 158, 209, 146, 127,
   62, 126, 56, 7, 70, 98, 97, 73, 73, 98, 4, 166, 106, 117, 42, 37, 1, 182,
   203, 178, 60, 30, 25, 45, 55, 41, 22, 64, 183, 197, 180, 97, 66, 39, 34,
   21, 22, 40, 56, 68, 44, 31, 31, 33, 85, 32, 62, 0, 20, 17, 35, 44, 19, 23,
   0, 147, 208, 73, 85, 14, 70, 9, 36, 37, 58, 78, 23, 47, 4, 11, 20, 3, 62,
   65, 18, 146, 197, 134, 111, 192, 223, 186, 160, 127, 125, 60, 171, 210,
   212, 207, 168, 174, 144, 191, 0, 148, 120, 35, 66, 29, 79, 182, 95, 71, 25,
   94, 114, 71, 166, 191, 147, 96, 24, 20, 47, 31, 39, 40, 112, 168, 209, 152,
   77, 39, 17, 12, 29, 51, 65, 74, 44, 15, 7, 21, 81, 66, 12, 40, 0, 13, 90,
   24, 19, 35, 1, 149, 47, 92, 91, 7, 68, 10, 44, 28, 28, 95, 0, 160, 106, 14,
   29, 12, 69, 43, 21, 245, 191, 122, 195, 229, 209, 207, 155, 97, 138, 254,
   175, 118, 247, 217, 217, 183, 94, 107, 115, 66, 102, 138, 13, 16, 156, 128,
   47, 22, 15, 44, 162, 138, 155, 165, 167, 172, 88, 3, 35, 31, 135, 134, 180,
   155, 192, 190, 165, 53, 9, 17, 28, 53, 75, 78, 70, 55, 30, 38, 57, 113, 7,
   33, 10, 31, 69, 23, 42, 13, 28, 8, 1, 55, 27, 59, 61, 27, 41, 33, 13, 37,
   40, 132, 143, 8, 22, 16, 44, 88, 84, 72, 152, 168, 172, 193, 147, 201, 178,
   101, 0, 224, 231, 250, 48, 3, 193, 233, 60, 78, 33, 159, 143, 212, 0, 50,
   0, 9, 11, 75, 39, 65, 67, 198, 200, 145, 56, 82, 79, 20, 50, 83, 39, 105,
   51, 70, 185, 210, 208, 189, 57, 15, 25, 41, 59, 72, 80, 112, 31, 36, 64,
   114, 97, 0, 24, 88, 149, 28, 15, 46, 16, 9, 54, 62, 1, 0, 110, 63, 16, 27,
   33, 56, 23, 17, 0, 43, 46, 66, 62, 12, 127, 167, 189, 57, 251, 227, 122, 2,
   0, 7, 68, 54, 95, 80, 247, 165, 212, 64, 111, 62, 219, 50, 127, 43, 83, 56,
   36, 17, 49, 66, 123, 111, 121, 145, 127, 175, 187, 84, 96, 95, 15, 33, 56,
   130, 130, 206, 168, 187, 206, 179, 144, 45, 43, 27, 24, 75, 110, 122, 35,
   29, 94, 73, 75, 134, 13, 34, 80, 74, 11, 121, 111, 45, 51, 29, 34, 190,
   213, 122, 17, 25, 25, 11, 110, 115, 33, 44, 66, 10, 43, 5, 66, 115, 133,
   57, 74, 244, 130, 0, 35, 16, 20, 89, 42, 60, 230, 219, 78, 197, 235, 139,
   147, 46, 10, 122, 117, 111, 90, 54, 56, 67, 82, 58, 16, 139, 145, 158, 210,
   103, 18, 37, 48, 35, 32, 37, 9, 33, 35, 120, 161, 175, 121, 68, 46, 14, 22,
   56, 103, 122, 71, 92, 87, 83, 0, 92, 127, 20, 44, 15, 118, 124, 121, 80,
   47, 83, 149, 140, 182, 65, 17, 26, 49, 65, 102, 186, 116, 22, 130, 13, 0,
   63, 40, 55, 24, 101, 10, 28, 130, 2, 21, 46, 8, 47, 58, 122, 251, 19, 106,
   208, 180, 212, 101, 6, 55, 92, 62, 125, 65, 89, 39, 34, 30, 2, 33, 111, 86,
   119, 213, 0, 163, 93, 17, 25, 35, 27, 27, 37, 35, 63, 105, 170, 125, 50,
   24, 2, 16, 43, 82, 78, 98, 97, 88, 18, 27, 0, 111, 124, 23, 34, 143, 215,
   156, 185, 159, 124, 110, 92, 161, 64, 22, 15, 59, 66, 65, 78, 57, 35, 22,
   42, 12, 88, 0, 23, 33, 119, 83, 43, 0, 18, 32, 54, 22, 50, 55, 3, 37, 157,
   64, 0, 11, 146, 22, 66, 99, 68, 92, 78, 70, 167, 26, 24, 7, 12, 44, 60, 5,
   87, 144, 52, 194, 170, 85, 25, 28, 30, 15, 25, 22, 21, 53, 124, 126, 82,
   54, 12, 28, 36, 56, 61, 103, 107, 135, 13, 25, 41, 24, 142, 45, 28, 19,
   243, 202, 186, 170, 235, 96, 111, 140, 80, 44, 8, 35, 21, 100, 20, 15, 9,
   0, 11, 37, 0, 31, 1, 103, 142, 100, 0, 12, 86, 40, 104, 62, 12, 20, 48, 50,
   51, 19, 53, 142, 19, 60, 84, 100, 67, 63, 93, 88, 2, 41, 22, 22, 28, 42,
   50, 40, 44, 35, 86, 120, 139, 119, 57, 18, 20, 8, 2, 18, 22, 7, 38, 51, 67,
   58, 60, 91, 38, 48, 71, 106, 114, 168, 248, 70, 20, 41, 0, 19, 56, 109, 48,
   209, 189, 156, 121, 251, 172, 202, 117, 130, 115, 62, 17, 81, 16, 43, 191,
   165, 67, 2, 18, 48, 178, 160, 114, 0, 143, 204, 187, 169, 43, 40, 36, 46,
   35, 78, 57, 6, 17, 86, 180, 30, 103, 28, 23, 115, 98, 62, 20, 65, 58, 55,
   72, 86, 82, 81, 80, 65, 23, 57, 92, 154, 64, 97, 78, 58, 40, 33, 43, 91,
   39, 25, 10, 32, 73, 72, 33, 49, 56, 12, 0, 78, 140, 190, 0, 92, 108, 18,
   13, 139, 196, 199, 215, 58, 74, 93, 215, 173, 96, 58, 85, 95, 21, 49, 82,
   0, 102, 219, 190, 150, 3, 151, 127, 158, 157, 0, 205, 221, 173, 188, 103,
   43, 48, 36, 72, 91, 123, 107, 56, 9, 93, 2, 88, 14, 81, 96, 73, 60, 20, 62,
   45, 54, 68, 41, 57, 31, 24, 60, 90, 8, 52, 83, 129, 195, 129, 93, 78, 88,
   131, 108, 104, 52, 29, 47, 89, 77, 51, 52, 84, 1, 3, 90, 70, 130, 1, 138,
   193, 204, 133, 159, 175, 184, 220, 77, 103, 182, 185, 209, 131, 104, 44,
   48, 45, 26, 74, 22, 0, 136, 223, 187, 180, 162, 68, 226, 57, 49, 169, 152,
   214, 214, 178, 21, 50, 44, 60, 53, 81, 103, 61, 38, 32, 22, 50, 2, 67, 172,
   183, 177, 27, 35, 23, 15, 14, 114, 80, 40, 45, 28, 12, 9, 86, 172, 175,
   183, 183, 125, 87, 125, 88, 130, 90, 59, 18, 65, 111, 78, 37, 34, 60, 1, 5,
   12, 30, 8, 17, 23, 124, 196, 223, 212, 146, 121, 198, 230, 230, 162, 212,
   131, 67, 35, 39, 14, 30, 17, 38, 43, 186, 192, 179, 218, 184, 216, 249, 30,
   22, 5, 43, 173, 165, 156, 0, 53, 5, 58, 52, 68, 103, 31, 41, 14, 16, 80,
   32, 14, 99, 248, 195, 86, 104, 188, 202, 87, 82, 182, 140, 89, 20, 18, 18,
   47, 48, 108, 225, 176, 198, 206, 98, 97, 117, 90, 71, 41, 14, 42, 99, 53,
   29, 28, 41, 7, 0, 31, 102, 26, 54, 9, 0, 156, 197, 227, 199, 203, 200, 235,
   194, 138, 202, 159, 14, 47, 32, 0, 52, 36, 12, 108, 150, 164, 179, 179, 98,
   157, 207, 1, 14, 0, 103, 118, 152, 159, 0, 34, 153, 96, 77, 127, 18, 4,
   103, 19, 26, 78, 64, 161, 229, 52, 0, 0, 68, 169, 130, 231, 72, 208, 197,
   124, 5, 46, 93, 47, 71, 84, 55, 229, 202, 204, 97, 113, 150, 154, 136, 138,
   132, 125, 52, 73, 22, 75, 41, 22, 0, 89, 75, 56, 73, 76, 136, 113, 226,
   181, 134, 176, 214, 214, 103, 150, 135, 213, 0, 47, 18, 5, 15, 53, 24, 28,
   63, 127, 198, 197, 171, 161, 219, 204, 30, 0, 117, 93, 110, 154, 52, 149,
   94, 103, 143, 145, 0, 127, 146, 125, 67, 179, 180, 107, 0, 9, 89, 195, 0,
   209, 154, 140, 195, 151, 235, 169, 2, 45, 49, 71, 154, 148, 176, 189, 218,
   107, 71, 188, 199, 151, 75, 53, 80, 74, 50, 68, 21, 68, 56, 4, 11, 32, 33,
   77, 135, 197, 192, 180, 147, 207, 91, 147, 141, 195, 182, 97, 100, 147, 1,
   44, 20, 12, 13, 8, 33, 32, 25, 30, 82, 115, 150, 210, 192, 223, 214, 0, 45,
   198, 177, 185, 220, 87, 92, 132, 52, 169, 186, 154, 166, 58, 41, 88, 45,
   20, 24, 87, 255, 82, 162, 190, 230, 212, 192, 193, 241, 83, 19, 46, 62,
   129, 180, 168, 182, 228, 237, 101, 7, 174, 191, 171, 164, 109, 67, 10, 59,
   60, 27, 31, 54, 15, 12, 85, 31, 137, 92, 173, 202, 192, 188, 235, 26, 91,
   221, 159, 208, 181, 24, 11, 19, 12, 18, 4, 14, 65, 25, 40, 26, 110, 76,
   134, 109, 203, 192, 160, 171, 67, 0, 247, 199, 212, 241, 166, 141, 141, 74,
   28, 217, 206, 191, 11, 64, 48, 103, 100, 60, 60, 230, 123, 49, 131, 122,
   174, 215, 0, 119, 104, 0, 107, 136, 87, 92, 116, 130, 79, 98, 249, 130,
   207, 213, 160, 0, 0, 19, 33, 58, 77, 41, 39, 63, 17, 18, 87, 72, 120, 89,
   74, 222, 221, 175, 182, 211, 223, 171, 225, 193, 136, 8, 12, 11, 15, 0, 13,
   1, 82, 25, 34, 45, 175, 80, 155, 62, 163, 162, 133, 0, 141, 172, 192, 200,
   71, 146, 248, 195, 140, 181, 240, 157, 216, 211, 2, 85, 113, 89, 26, 160,
   224, 168, 153, 97, 81, 193, 42, 201, 3, 209, 190, 90, 71, 129, 89, 63, 63,
   106, 4, 4, 210, 177, 214, 156, 236, 38, 11, 7, 25, 95, 96, 41, 49, 61, 64,
   129, 54, 124, 103, 96, 95, 185, 18, 147, 208, 184, 160, 185, 168, 209, 0,
   15, 22, 7, 21, 26, 27, 21, 4, 30, 24, 141, 165, 188, 109, 45, 126, 121,
   143, 180, 186, 193, 231, 164, 68, 8, 164, 193, 113, 124, 141, 175, 168,
   227, 126, 36, 52, 0, 160, 136, 187, 203, 106, 170, 82, 59, 56, 203, 232,
   170, 152, 98, 42, 44, 79, 98, 132, 57, 11, 88, 198, 193, 173, 86, 165, 246,
   63, 33, 12, 73, 56, 42, 41, 49, 122, 124, 109, 105, 81, 84, 22, 72, 65,
   148, 195, 220, 184, 211, 135, 202, 52, 0, 14, 32, 46, 47, 44, 25, 35, 29,
   25, 38, 68, 8, 57, 78, 89, 135, 44, 139, 165, 190, 170, 202, 89, 199, 193,
   200, 136, 128, 155, 98, 36, 197, 234, 58, 0, 37, 72, 114, 239, 161, 53, 0,
   19, 0, 63, 163, 154, 184, 150, 89, 98, 104, 102, 123, 74, 7, 89, 6, 126,
   232, 56, 17, 62, 198, 117, 105, 67, 37, 28, 43, 50, 28, 131, 141, 47, 28,
   35, 91, 70, 35, 16, 22, 206, 188, 65, 84, 200, 142, 109, 26, 37, 21, 6, 29,
   5, 30, 15, 17, 31, 11, 11, 43, 62, 38, 8, 20, 94, 102, 178, 161, 156, 146,
   235, 173, 157, 135, 111, 98, 24, 39, 35, 38, 212, 115, 0, 48, 63, 20, 238,
   0, 5, 12, 1, 18, 34, 134, 143, 207, 62, 85, 97, 125, 154, 64, 0, 52, 9, 35,
   58, 100, 22, 19, 25, 102, 214, 49, 46, 32, 55, 52, 46, 49, 140, 137, 59,
   23, 8, 87, 110, 0, 23, 2, 26, 162, 37, 89, 49, 190, 118, 79, 3, 11, 17, 21,
   21, 11, 21, 4, 46, 17, 23, 45, 51, 46, 26, 54, 0, 13, 254, 233, 184, 208,
   150, 240, 168, 144, 83, 8, 4, 236, 204, 6, 184, 0, 34, 59, 52, 15, 0, 25,
   45, 3, 11, 55, 28, 104, 118, 182, 27, 68, 94, 158, 130, 10, 118, 59, 30,
   41, 63, 21, 18, 43, 44, 11, 96, 7, 44, 51, 51, 45, 51, 53, 109, 147, 68, 0,
   1, 0, 91, 65, 7, 10, 75, 111, 74, 24, 199, 143, 141, 99, 0, 10, 47, 12, 22,
   25, 16, 16, 38, 10, 5, 60, 89, 8, 21, 68, 18, 55, 235, 219, 255, 195, 107,
   255, 211, 192, 122, 11, 163, 166, 178, 23, 76, 35, 33, 79, 25, 48, 14, 44,
   13, 13, 19, 7, 26, 60, 71, 83, 27, 51, 133, 123, 0, 93, 104, 19, 39, 32,
   32, 31, 0, 2, 21, 104, 6, 140, 43, 35, 32, 42, 33, 54, 136, 144, 19, 24, 6,
   0, 2, 47, 4, 19, 69, 88, 98, 132, 219, 199, 97, 116, 83, 0, 18, 18, 8, 20,
   27, 12, 14, 13, 20, 67, 4, 12, 47, 9, 25, 16, 76, 172, 85, 23, 25, 98, 196,
   184, 74, 19, 177, 141, 218, 242, 91, 187, 128, 85, 107, 216, 92, 0, 16, 15,
   11, 57, 85, 18, 53, 146, 23, 86, 85, 3, 88, 58, 32, 51, 29, 16, 10, 42,
   200, 155, 168, 36, 97, 21, 22, 27, 29, 49, 42, 29, 152, 157, 26, 11, 15,
   10, 12, 12, 10, 25, 80, 80, 77, 27, 95, 161, 115, 104, 21, 17, 12, 15, 40,
   14, 18, 13, 4, 115, 212, 109, 192, 33, 30, 55, 46, 27, 15, 79, 32, 222, 62,
   8, 6, 143, 168, 160, 172, 178, 179, 85, 85, 65, 205, 120, 55, 137, 209, 11,
   14, 0, 77, 120, 34, 39, 75, 49, 41, 52, 18, 43, 91, 67, 14, 15, 6, 22, 26,
   107, 63, 28, 0, 39, 32, 24, 29, 20, 32, 31, 39, 45, 152, 155, 15, 13, 0, 0,
   14, 12, 4, 34, 62, 53, 93, 56, 73, 6, 39, 120, 110, 0, 16, 0, 0, 9, 13, 0,
   32, 188, 40, 207, 243, 226, 0, 6, 14, 20, 96, 47, 24, 179, 248, 43, 6, 86,
   220, 150, 179, 207, 163, 162, 138, 204, 210, 74, 43, 198, 166, 190, 211,
   174, 7, 52, 39, 34, 56, 30, 31, 11, 51, 63, 55, 59, 87, 83, 70, 44, 50, 27,
   32, 34, 35, 32, 38, 31, 33, 26, 59, 57, 45, 52, 67, 73, 0, 0, 6, 3, 11, 8,
   2, 5, 44, 35, 170, 51, 76, 58, 141, 82, 15, 0, 0, 16, 33, 60, 5, 0, 60,
   174, 197, 105, 226, 244, 242, 0, 0, 82, 65, 27, 16, 45, 164, 0, 0, 131,
   113, 113, 72, 215, 191, 205, 174, 199, 208, 137, 0, 207, 175, 234, 210,
   240, 58, 35, 26, 34, 25, 20, 12, 56, 60, 18, 22, 22, 26, 46, 66, 94, 79,
   86, 30, 16, 18, 29, 22, 80, 77, 57, 30, 56, 38, 45, 70, 76, 91, 2, 4, 9,
   18, 1, 3, 4, 0, 0, 125, 99, 26, 2, 0, 0, 7, 6, 6, 28, 96, 62, 4, 22, 31,
   29, 249, 249, 209, 164, 236, 239, 164, 206, 174, 7, 66, 168, 0, 7, 140, 0,
   193, 239, 50, 218, 185, 151, 207, 216, 173, 189, 75, 220, 203, 168, 224,
   205, 128, 0, 39, 20, 3, 42, 51, 52, 44, 23, 12, 10, 10, 20, 41, 10, 33, 34,
   62, 68, 25, 18, 78, 86, 85, 69, 81, 49, 37, 35, 83, 91}, 
  {191, 213, 194, 155, 143, 99, 147, 188, 175, 161, 193, 194, 164, 191, 186,
   177, 193, 165, 185, 211, 188, 146, 47, 18, 31, 38, 36, 53, 43, 33, 44, 22, 19,
   26, 17, 24, 8, 107, 187, 210, 190, 135, 94, 97, 74, 35, 17, 56, 84, 106, 98,
   77, 67, 46, 40, 32, 56, 66, 64, 47, 41, 39, 36, 46, 42, 55, 33, 33, 37, 38, 30,
   36, 39, 44, 33, 13, 15, 17, 26, 36, 36, 27, 21, 19, 28, 21, 27, 38, 49, 39, 167,
   191, 98, 87, 172, 136, 150, 212, 170, 86, 142, 196, 209, 172, 141, 112, 200, 151, 136,
   207, 190, 198, 173, 106, 50, 25, 14, 7, 28, 40, 35, 39, 33, 24, 29, 24, 1, 106, 193, 199,
   195, 198, 188, 195, 189, 106, 56, 94, 118, 90, 73, 61, 63, 70, 52, 45, 65, 56,
   45, 42, 35, 29, 38, 51, 54, 59, 36, 47, 50, 45, 38, 38, 58, 60, 47, 22, 26, 25,
   30, 31, 24, 18, 19, 23, 27, 19, 37, 65, 39, 40, 141, 129, 47, 175, 202, 217,
   194, 171, 136, 53, 53, 136, 119, 184, 205, 84, 154, 214, 199, 208, 182, 162, 190, 170,
   128, 54, 88, 75, 26, 36, 41, 42, 46, 38, 47, 33, 13, 63, 131, 160, 180, 195,
   188, 177, 163, 125, 88, 114, 106, 90, 39, 48, 78, 72, 46, 41, 57, 51, 39, 32,
   31, 30, 18, 27, 27, 42, 47, 26, 48, 38, 33, 42, 64, 71, 62, 28, 35, 36, 40, 28,
   21, 18, 22, 26, 31, 25, 46, 72, 34, 34, 112, 12, 61, 222, 206, 197, 210, 188,
   62, 0, 12, 27, 8, 115, 145, 49, 23, 191, 212, 197, 185, 182, 141, 110, 88,
   58, 141, 178, 134, 60, 46, 44, 34, 30, 39, 32, 27, 16, 57, 133, 162, 167, 144,
   126, 114, 60, 94, 105, 107, 59, 32, 47, 69, 70, 41, 35, 50, 51, 34, 36, 33,
   35, 7, 105, 100, 16, 23, 16, 25, 38, 17, 37, 63, 72, 68, 31, 40, 43, 48, 32, 14,
   15, 23, 24, 30, 31, 47, 64, 38, 33, 150, 140, 127, 150, 113, 187, 219, 167,
   122, 101, 34, 8, 20, 149, 102, 99, 81, 36, 101, 220, 194, 209, 143, 149, 148,
   130, 74, 143, 172, 164, 125, 76, 25, 24, 46, 18, 20, 20, 27, 106, 142, 170,
   147, 87, 73, 81, 81, 96, 87, 44, 55, 73, 72, 53, 39, 39, 58, 35, 27, 80, 93, 5,
   88, 177, 188, 96, 0, 44, 25, 29, 28, 34, 66, 76, 72, 36, 41, 41, 51, 41, 19,
   20, 26, 25, 27, 25, 39, 49, 69, 57, 199, 202, 219, 136, 58, 124, 211, 173,
   197, 189, 176, 106, 164, 185, 138, 140, 196, 201, 165, 209, 216, 167, 35,
   141, 173, 178, 124, 63, 96, 111, 118, 126, 61, 27, 40, 19, 25, 28, 14, 125,
   161, 200, 194, 97, 77, 73, 70, 65, 63, 52, 62, 74, 71, 48, 50, 50, 36, 38, 26,
   104, 172, 116, 127, 173, 188, 141, 32, 62, 64, 65, 28, 24, 60, 81, 83, 42, 36,
   42, 58, 52, 39, 25, 22, 23, 24, 17, 25, 31, 54, 55, 157, 144, 181, 220, 140,
   171, 198, 139, 198, 199, 147, 170, 209, 189, 193, 142, 74, 140, 203, 146,
   183, 191, 115, 69, 150, 175, 138, 114, 85, 45, 103, 112, 92, 37, 26, 42,
   31, 26, 13, 98, 175, 201, 216, 124, 56, 65, 62, 68, 66, 51, 79, 75, 71, 81, 56,
   35, 46, 52, 38, 65, 181, 201, 175, 168, 178, 165, 131, 140, 116, 72, 58, 57,
   60, 87, 93, 60, 51, 59, 71, 65, 49, 30, 26, 31, 36, 29, 29, 25, 52, 51, 56, 118,
   198, 204, 221, 214, 203, 187, 189, 208, 123, 145, 221, 222, 218, 97, 11,
   192, 179, 173, 144, 181, 229, 146, 96, 148, 135, 149, 165, 141, 155, 192,
   181, 64, 50, 169, 118, 28, 5, 51, 203, 187, 200, 189, 130, 63, 54, 60, 39,
   77, 89, 85, 95, 97, 63, 54, 49, 44, 50, 29, 143, 201, 181, 211, 204, 203, 190,
   162, 123, 74, 56, 73, 75, 93, 98, 82, 80, 84, 84, 75, 64, 57, 61, 59, 56, 49,
   43, 31, 54, 49, 22, 79, 198, 167, 168, 182, 194, 183, 206, 211, 90, 67,
   130, 188, 212, 180, 144, 172, 177, 210, 158, 80, 118, 124, 155, 152, 184,
   173, 190, 184, 197, 207, 213, 65, 57, 163, 128, 46, 96, 87, 198, 187, 173,
   212, 196, 158, 109, 60, 50, 77, 105, 78, 69, 67, 71, 74, 64, 60, 47, 22, 69, 136,
   155, 199, 214, 215, 189, 163, 151, 110, 66, 69, 79, 94, 98, 103, 108, 90,
   91, 82, 70, 60, 60, 55, 52, 48, 37, 33, 34, 32, 114, 9, 43, 63, 106, 147, 168,
   166, 178, 208, 68, 112, 107, 197, 167, 180, 186, 180, 139, 175, 106, 204,
   130, 116, 120, 152, 150, 141, 160, 151, 167, 185, 191, 65, 50, 116, 39, 50,
   128, 113, 134, 190, 192, 186, 196, 189, 194, 97, 15, 48, 100, 130, 99, 34, 84,
   79, 86, 60, 33, 21, 25, 74, 119, 171, 195, 210, 189, 173, 150, 120, 57, 51, 61,
   81, 96, 114, 128, 110, 99, 82, 60, 57, 49, 37, 39, 30, 22, 32, 30, 28, 96,
   65, 52, 40, 53, 89, 99, 153, 172, 199, 72, 136, 204, 204, 204, 145, 181,
   180, 104, 119, 77, 62, 173, 108, 102, 96, 63, 62, 49, 92, 174, 160, 158,
   48, 76, 171, 91, 105, 112, 161, 12, 79, 186, 183, 170, 171, 197, 186, 60,
   125, 172, 199, 203, 79, 45, 79, 91, 77, 22, 26, 26, 26, 70, 144, 188, 182, 123,
   80, 74, 55, 50, 34, 26, 49, 70, 96, 124, 124, 111, 84, 52, 30, 33, 25, 16, 17,
   34, 34, 27, 28, 94, 59, 33, 27, 70, 72, 107, 175, 145, 103, 57, 197, 169,
   201, 194, 167, 109, 170, 177, 133, 120, 82, 155, 113, 84, 62, 67, 42, 8,
   76, 180, 178, 181, 30, 105, 184, 160, 182, 187, 210, 111, 116, 174, 174,
   154, 186, 204, 208, 202, 163, 116, 153, 171, 137, 50, 72, 72, 75, 85, 38, 34,
   33, 28, 34, 81, 82, 58, 62, 81, 95, 68, 43, 3, 20, 40, 72, 111, 131, 117, 80,
   45, 33, 34, 29, 32, 28, 36, 39, 28, 24, 109, 59, 30, 11, 34, 38, 73, 114, 37, 46,
   74, 166, 107, 29, 146, 146, 136, 145, 218, 162, 181, 199, 173, 122, 63,
   80, 42, 42, 32, 89, 167, 180, 167, 31, 103, 208, 178, 198, 209, 234, 164,
   114, 132, 155, 148, 188, 190, 178, 198, 194, 140, 172, 192, 158, 92, 71, 84,
   96, 82, 74, 33, 62, 59, 34, 49, 56, 24, 56, 84, 98, 93, 51, 10, 12, 23, 55, 93,
   130, 128, 94, 43, 37, 35, 32, 34, 31, 40, 42, 32, 27, 117, 57, 11, 19, 17, 19,
   21, 81, 52, 43, 77, 52, 21, 20, 22, 190, 199, 165, 197, 186, 201, 197, 177,
   100, 55, 30, 29, 108, 91, 63, 88, 86, 98, 40, 79, 188, 196, 182, 193, 182, 177,
   84, 87, 124, 66, 105, 195, 165, 152, 190, 162, 169, 163, 124, 122, 91, 77, 79,
   89, 89, 76, 85, 97, 60, 83, 63, 38, 28, 49, 80, 103, 89, 30, 33, 51, 70, 84, 122,
   130, 98, 49, 41, 43, 45, 29, 29, 41, 33, 25, 24, 106, 68, 27, 30, 35, 27, 21, 29,
   34, 31, 95, 37, 51, 29, 29, 144, 226, 209, 203, 183, 68, 163, 185, 191, 150,
   59, 63, 117, 54, 73, 57, 142, 171, 26, 76, 156, 162, 201, 148, 138, 195,
   149, 123, 86, 22, 116, 175, 174, 170, 203, 139, 83, 82, 167, 159, 77,
   75, 58, 47, 123, 128, 89, 128, 126, 106, 75, 26, 37, 29, 52, 86, 96, 65, 61,
   79, 87, 72, 102, 123, 105, 58, 75, 67, 72, 70, 60, 39, 32, 21, 21, 52, 84, 27,
   32, 29, 21, 28, 33, 28, 26, 88, 38, 31, 67, 42, 19, 118, 165, 159, 181, 157,
   161, 197, 180, 131, 121, 115, 35, 125, 174, 145, 181, 188, 26, 120, 195,
   186, 188, 174, 156, 133, 172, 180, 169, 38, 116, 161, 182, 189, 193, 173,
   93, 97, 169, 167, 84, 40, 54, 21, 110, 171, 125, 132, 187, 164, 79, 24, 33,
   26, 18, 49, 83, 78, 77, 124, 161, 139, 146, 149, 129, 74, 81, 73, 90, 87,
   92, 74, 54, 21, 18, 22, 91, 34, 37, 27, 27, 24, 50, 42, 35, 96, 33, 11, 48, 52,
   9, 180, 226, 188, 208, 221, 149, 152, 84, 58, 64, 73, 131, 171, 172, 112,
   103, 133, 33, 34, 87, 28, 100, 217, 99, 118, 182, 179, 116, 71, 89, 170,
   186, 191, 148, 186, 183, 137, 80, 130, 74, 52, 45, 33, 65, 150, 158, 128, 195,
   182, 121, 93, 88, 14, 6, 19, 37, 56, 100, 159, 179, 212, 205, 197, 211,
   95, 39, 73, 58, 66, 65, 68, 64, 60, 51, 54, 46, 39, 27, 57, 80, 52, 29, 37, 56,
   76, 107, 82, 60, 43, 28, 107, 192, 103, 99, 106, 124, 141, 137, 49, 76, 34,
   72, 125, 152, 114, 9, 44, 79, 98, 198, 171, 180, 212, 197, 191, 178, 192,
   143, 104, 36, 87, 212, 189, 200, 210, 186, 191, 138, 128, 92, 86, 65, 36,
   29, 119, 186, 183, 188, 183, 170, 140, 90, 21, 3, 0, 7, 26, 67, 107, 140,
   179, 194, 155, 199, 174, 38, 20, 46, 53, 46, 44, 64, 51, 51, 39, 43, 33, 20,
   38, 80, 41, 73, 32, 73, 69, 107, 56, 86, 85, 43, 56, 94, 112, 78, 69, 193, 142,
   182, 150, 21, 5, 36, 81, 150, 104, 28, 54, 95, 25, 91, 254, 167, 244, 89,
   236, 172, 153, 192, 232, 137, 9, 87, 200, 206, 229, 183, 226, 240, 198, 120,
   87, 45, 40, 19, 135, 200, 206, 211, 175, 144, 62, 8, 0, 28, 102, 108, 92,
   127, 124, 133, 110, 173, 167, 163, 214, 175, 82, 78, 69, 53, 24, 40, 51, 52,
   20, 35, 36, 33, 3, 96, 33, 154, 116, 66, 67, 104, 131, 73, 67, 47, 41, 54,
   58, 88, 114, 159, 243, 194, 177, 208, 50, 0, 49, 33, 14, 70, 60, 59, 31, 1,
   48, 192, 190, 60, 255, 112, 145, 193, 123, 17, 182, 164, 94, 181, 67, 85,
   200, 195, 205, 197, 83, 32, 32, 54, 91, 172, 191, 182, 203, 158, 61, 0, 4,
   68, 162, 203, 176, 177, 179, 169, 110, 166, 222, 196, 193, 188, 142, 93,
   86, 54, 26, 23, 53, 55, 63, 25, 12, 14, 31, 96, 56, 169, 149, 137, 61, 43,
   115, 113, 101, 49, 37, 10, 68, 172, 124, 26, 163, 207, 234, 185, 134, 218,
   160, 198, 205, 42, 36, 57, 26, 32, 27, 51, 243, 56, 193, 183, 152, 69, 11,
   143, 189, 184, 201, 212, 212, 102, 194, 201, 159, 67, 109, 130, 85, 56, 70,
   80, 83, 111, 151, 162, 99, 52, 48, 30, 89, 191, 182, 183, 178, 174, 116,
   136, 159, 190, 207, 193, 159, 134, 89, 31, 24, 23, 59, 62, 38, 50, 18, 72, 177,
   147, 47, 161, 125, 191, 149, 52, 27, 57, 79, 53, 44, 85, 149, 163, 90, 86,
   167, 145, 197, 225, 90, 177, 204, 199, 131, 0, 36, 53, 32, 22, 7, 128, 197,
   74, 84, 181, 106, 85, 36, 18, 103, 186, 214, 176, 232, 224, 191, 189, 236,
   82, 146, 156, 132, 92, 67, 82, 79, 103, 119, 123, 94, 74, 84, 81, 11, 76, 158,
   201, 185, 105, 122, 97, 124, 124, 145, 189, 137, 143, 74, 29, 34, 38, 63,
   64, 38, 32, 22, 33, 202, 194, 55, 176, 138, 188, 226, 215, 153, 29, 114,
   46, 26, 164, 182, 171, 156, 134, 209, 207, 187, 162, 74, 110, 205, 142,
   3, 13, 36, 56, 25, 24, 12, 38, 174, 90, 20, 219, 129, 114, 26, 55, 30, 2,
   55, 0, 200, 197, 194, 177, 188, 176, 69, 98, 162, 139, 115, 45, 56, 127,
   149, 153, 135, 79, 101, 120, 93, 65, 175, 196, 172, 161, 195, 92, 75, 74,
   47, 59, 75, 72, 45, 33, 35, 34, 65, 67, 36, 51, 65, 56, 44, 142, 22, 125, 92,
   230, 202, 218, 198, 85, 40, 54, 35, 161, 227, 161, 34, 121, 175, 102, 80,
   21, 90, 44, 143, 222, 122, 2, 43, 52, 36, 9, 63, 190, 119, 97, 27, 159, 163,
   233, 123, 23, 32, 16, 56, 64, 47, 63, 101, 177, 168, 183, 52, 143, 222,
   189, 114, 77, 145, 147, 149, 169, 129, 132, 86, 138, 125, 107, 174, 171,
   138, 162, 188, 100, 43, 55, 50, 28, 52, 42, 41, 36, 34, 34, 67, 66, 22, 49, 105,
   141, 62, 128, 213, 188, 156, 122, 232, 205, 195, 148, 46, 53, 32, 107,
   209, 158, 30, 183, 232, 145, 0, 106, 115, 101, 24, 154, 144, 168, 70,
   44, 0, 58, 180, 193, 138, 85, 50, 125, 144, 206, 196, 115, 170, 136, 5,
   192, 37, 65, 75, 69, 165, 155, 200, 126, 196, 232, 152, 177, 140, 89, 136,
   109, 101, 105, 98, 154, 142, 85, 82, 97, 102, 106, 135, 133, 37, 45, 45, 37,
   32, 42, 30, 33, 36, 24, 25, 37, 25, 16, 11, 29, 121, 163, 161, 183, 199, 204,
   54, 119, 255, 119, 208, 56, 41, 79, 180, 105, 45, 188, 211, 149, 4,
   150, 171, 125, 171, 166, 144, 130, 123, 59, 204, 110, 67, 155, 235, 88,
   54, 138, 101, 184, 222, 151, 152, 155, 115, 198, 177, 156, 147, 107, 64,
   52, 186, 153, 217, 179, 254, 108, 75, 126, 141, 60, 101, 59, 108, 122,
   126, 74, 51, 38, 61, 52, 146, 140, 57, 34, 35, 38, 27, 26, 33, 34, 26, 29, 41,
   37, 59, 15, 27, 35, 28, 129, 170, 186, 215, 232, 184, 106, 25, 115, 195,
   64, 52, 45, 102, 26, 34, 104, 163, 48, 79, 205, 156, 199, 170, 162, 195,
   192, 76, 139, 215, 173, 126, 151, 51, 102, 35, 139, 237, 180, 196, 169,
   94, 33, 105, 88, 121, 147, 101, 61, 42, 38, 16, 50, 235, 208, 109, 93, 115,
   180, 31, 77, 90, 89, 73, 64, 49, 64, 47, 70, 84, 69, 77, 113, 45, 40, 32, 33, 28,
   27, 50, 45, 24, 31, 39, 40, 159, 54, 0, 25, 0, 0, 91, 175, 182, 205, 175, 91, 0,
   166, 238, 65, 40, 25, 16, 99, 2, 35, 64, 4, 76, 56, 244, 145, 209, 190,
   191, 233, 223, 181, 211, 224, 120, 10, 79, 109, 59, 95, 230, 195, 146,
   180, 77, 32, 32, 110, 167, 91, 56, 45, 39, 24, 51, 41, 93, 115, 119, 74, 219,
   169, 96, 92, 87, 64, 32, 38, 23, 26, 32, 77, 103, 98, 56, 60, 48, 50, 48, 33, 36,
   37, 62, 54, 29, 33, 39, 42, 193, 117, 0, 4, 9, 26, 0, 132, 215, 153, 210, 0, 58,
   225, 83, 71, 45, 35, 11, 43, 62, 12, 105, 31, 46, 12, 94, 235, 172, 177, 195,
   219, 227, 181, 180, 224, 154, 77, 50, 110, 53, 56, 197, 193, 110, 86, 28, 68,
   46, 188, 174, 133, 32, 46, 34, 41, 45, 31, 73, 109, 97, 142, 219, 206, 136,
   114, 79, 41, 86, 66, 37, 48, 43, 76, 93, 86, 103, 56, 89, 53, 55, 34, 35, 38, 65,
   61, 42, 60, 21, 33, 152, 83, 13, 11, 2, 36, 0, 153, 198, 222, 67, 6, 150, 180,
   139, 54, 58, 37, 13, 28, 70, 60, 20, 14, 79, 154, 91, 131, 173, 137, 168, 111,
   215, 185, 161, 207, 215, 157, 89, 71, 83, 64, 100, 154, 97, 1, 111, 117, 26,
   78, 164, 91, 17, 20, 73, 86, 72, 0, 85, 130, 102, 166, 254, 195, 124, 155, 46,
   50, 90, 75, 50, 30, 35, 75, 89, 101, 81, 100, 66, 86, 50, 40, 36, 39, 69, 66,
   46, 76, 36, 32, 100, 39, 14, 16, 12, 26, 45, 226, 197, 254, 9, 33, 227, 167, 133,
   116, 45, 47, 13, 40, 73, 41, 42, 43, 118, 142, 169, 218, 161, 215, 254, 230,
   214, 197, 238, 238, 216, 165, 100, 71, 76, 53, 137, 169, 10, 141, 82, 122,
   67, 8, 129, 97, 34, 74, 122, 99, 55, 41, 150, 107, 105, 44, 193, 190, 188,
   189, 61, 20, 43, 53, 77, 56, 10, 109, 170, 136, 113, 84, 85, 51, 40, 36, 36, 47,
   74, 76, 53, 68, 34, 37, 137, 104, 18, 10, 28, 25, 24, 0, 32, 14, 29, 87, 144,
   113, 99, 41, 73, 34, 41, 33, 60, 59, 17, 28, 76, 131, 74, 84, 93, 72, 44, 163,
   190, 72, 88, 237, 177, 167, 123, 85, 103, 17, 164, 228, 166, 128, 64,
   129, 54, 11, 73, 94, 98, 82, 78, 102, 2, 170, 109, 113, 43, 46, 6, 186, 211,
   182, 68, 36, 24, 37, 56, 53, 30, 68, 191, 203, 179, 89, 67, 51, 43, 32, 34, 52,
   68, 81, 59, 47, 37, 39, 97, 43, 71, 14, 29, 19, 35, 45, 31, 34, 9, 161, 217, 75,
   85, 15, 91, 26, 50, 40, 59, 83, 26, 48, 25, 28, 34, 6, 63, 70, 21, 147, 201,
   134, 109, 194, 227, 189, 160, 124, 129, 60, 169, 212, 216, 210, 168, 171,
   140, 189, 0, 153, 129, 42, 63, 23, 75, 180, 94, 76, 34, 101, 111, 65, 162,
   183, 148, 106, 32, 21, 46, 38, 35, 32, 113, 178, 217, 153, 76, 46, 33, 35,
   42, 54, 70, 80, 52, 22, 23, 44, 86, 78, 26, 57, 8, 26, 101, 30, 24, 47, 15, 166,
   60, 105, 102, 13, 87, 31, 58, 34, 33, 103, 3, 163, 125, 35, 43, 18, 74, 51,
   27, 248, 190, 125, 197, 238, 217, 210, 160, 99, 137, 255, 177, 127, 255,
   220, 222, 185, 94, 107, 113, 71, 117, 149, 19, 11, 156, 128, 45, 27, 30,
   55, 168, 133, 156, 166, 171, 179, 88, 0, 25, 17, 136, 135, 184, 162, 192,
   184, 155, 39, 21, 33, 38, 57, 73, 73, 62, 45, 43, 41, 63, 126, 21, 46, 18, 45,
   84, 31, 48, 26, 42, 21, 9, 69, 42, 67, 74, 45, 56, 43, 22, 43, 47, 135, 156,
   26, 37, 26, 53, 94, 91, 75, 155, 170, 178, 203, 158, 210, 183, 102, 3, 226,
   237, 255, 59, 12, 198, 234, 55, 79, 38, 167, 160, 227, 7, 48, 0, 10, 16, 83,
   56, 80, 77, 196, 201, 150, 62, 90, 81, 14, 39, 62, 40, 110, 57, 78, 187, 204,
   197, 168, 57, 15, 31, 53, 68, 79, 90, 118, 36, 42, 71, 127, 116, 13, 30, 92,
   155, 28, 22, 59, 35, 24, 60, 66, 7, 0, 125, 79, 33, 40, 42, 63, 32, 25, 11, 59,
   63, 79, 71, 19, 136, 175, 188, 62, 255, 237, 133, 10, 5, 13, 67, 59, 99, 90,
   255, 173, 220, 70, 113, 65, 224, 67, 146, 60, 93, 60, 38, 20, 54, 83, 142,
   128, 131, 149, 139, 184, 190, 86, 98, 90, 8, 21, 68, 139, 133, 208, 170,
   182, 199, 167, 139, 36, 45, 45, 39, 88, 130, 139, 43, 36, 105, 85, 89,
   145, 20, 32, 80, 70, 22, 133, 125, 56, 58, 27, 34, 186, 220, 132, 34, 41, 32,
   14, 113, 115, 40, 54, 83, 26, 50, 8, 69, 115, 132, 58, 74, 250, 140, 10,
   50, 26, 19, 90, 42, 66, 240, 231, 93, 207, 239, 144, 155, 62, 25, 139,
   132, 127, 94, 59, 64, 83, 97, 75, 31, 155, 159, 168, 219, 111, 23, 40, 47,
   36, 46, 47, 18, 41, 40, 123, 160, 176, 123, 55, 46, 31, 23, 44, 96, 110, 83,
   108, 101, 95, 7, 93, 128, 17, 42, 7, 132, 136, 130, 81, 48, 80, 147, 132, 188,
   79, 33, 46, 60, 66, 107, 182, 122, 36, 146, 33, 10, 64, 45, 51, 27, 102, 10,
   31, 139, 16, 34, 56, 11, 48, 58, 125, 255, 33, 119, 218, 188, 220, 106, 12,
   65, 100, 73, 122, 73, 97, 44, 40, 40, 10, 44, 108, 92, 129, 224, 0, 164, 96,
   19, 31, 41, 37, 38, 40, 36, 66, 107, 176, 130, 32, 23, 26, 20, 35, 85, 74, 108,
   101, 97, 32, 36, 3, 111, 120, 18, 32, 152, 229, 165, 188, 159, 120, 105, 90,
   170, 80, 33, 29, 67, 63, 65, 75, 66, 51, 33, 56, 20, 85, 0, 20, 35, 125, 85, 48,
   1, 22, 40, 57, 24, 56, 57, 8, 42, 161, 72, 0, 15, 149, 24, 70, 103, 67, 92, 69,
   74, 170, 28, 28, 11, 11, 44, 51, 10, 100, 157, 58, 198, 174, 87, 26, 33, 43,
   28, 31, 26, 25, 55, 125, 125, 52, 48, 40, 39, 37, 76, 79, 101, 102, 151, 30,
   28, 45, 28, 149, 47, 35, 35, 255, 205, 190, 174, 242, 98, 118, 148, 96, 59, 22,
   49, 23, 95, 14, 23, 25, 8, 25, 51, 0, 26, 0, 105, 146, 102, 0, 14, 88, 42,
   102, 64, 16, 22, 49, 52, 53, 21, 51, 143, 19, 61, 88, 100, 66, 61, 90, 89, 2, 42,
   26, 22, 27, 40, 47, 49, 60, 44, 88, 121, 141, 119, 52, 27, 36, 17, 4, 19, 24,
   7, 33, 66, 45, 53, 84, 86, 19, 49, 70, 98, 104, 180, 255, 79, 34, 50, 0, 19,
   57, 121, 59, 218, 203, 165, 123, 251, 173, 223, 134, 144, 118, 63, 22, 84,
   17, 64, 208, 179, 70, 3, 23, 51, 179, 164, 114, 0, 145, 208, 190, 169,
   40, 44, 36, 44, 37, 82, 60, 6, 14, 82, 178, 29, 108, 37, 30, 112, 92, 58, 18,
   64, 63, 64, 79, 83, 76, 77, 72, 66, 33, 65, 93, 153, 71, 93, 70, 59, 50, 41, 44,
   90, 46, 41, 33, 45, 76, 77, 39, 57, 63, 28, 13, 83, 152, 204, 10, 105, 121,
   29, 19, 144, 208, 213, 232, 71, 87, 104, 221, 192, 117, 72, 91, 100, 29,
   55, 85, 13, 123, 233, 196, 155, 11, 157, 130, 157, 160, 0, 214, 229, 176,
   193, 105, 42, 51, 38, 81, 99, 126, 112, 58, 9, 93, 0, 93, 29, 92, 102, 68, 60,
   20, 60, 50, 69, 79, 47, 52, 32, 25, 64, 97, 8, 46, 73, 115, 196, 130, 97,
   85, 88, 125, 98, 90, 64, 45, 57, 93, 75, 46, 44, 74, 14, 6, 96, 83, 144, 14,
   146, 207, 219, 141, 165, 188, 198, 233, 85, 117, 197, 193, 222, 149, 119,
   54, 57, 51, 33, 77, 35, 10, 151, 233, 196, 186, 169, 71, 229, 59, 55, 179,
   163, 223, 219, 179, 24, 52, 50, 70, 64, 90, 108, 62, 33, 33, 27, 58, 19, 82, 182,
   181, 172, 28, 40, 31, 32, 29, 124, 78, 41, 50, 34, 20, 11, 80, 161, 154, 184,
   188, 131, 95, 127, 82, 119, 69, 59, 18, 71, 123, 87, 44, 44, 66, 6, 11, 19,
   43, 27, 32, 29, 128, 202, 223, 219, 159, 140, 213, 236, 234, 168, 212, 146,
   83, 52, 52, 23, 37, 26, 46, 58, 202, 209, 192, 227, 191, 225, 255, 29, 27, 9,
   53, 184, 173, 164, 3, 52, 10, 62, 62, 79, 111, 39, 47, 16, 19, 85, 49, 33, 116,
   255, 199, 88, 107, 193, 219, 106, 99, 192, 144, 101, 29, 21, 20, 49, 43,
   101, 213, 188, 207, 209, 100, 99, 112, 83, 59, 36, 5, 44, 117, 68, 42, 48, 58,
   15, 6, 42, 114, 40, 65, 16, 0, 156, 193, 238, 211, 217, 211, 242, 192, 138,
   198, 166, 24, 64, 48, 7, 55, 39, 12, 115, 160, 181, 195, 186, 101, 160,
   207, 0, 15, 0, 109, 128, 164, 174, 7, 33, 154, 96, 83, 137, 30, 19, 113, 23, 31,
   86, 80, 176, 246, 67, 12, 0, 73, 177, 146, 246, 89, 223, 213, 138, 15,
   55, 101, 52, 74, 83, 56, 243, 212, 213, 105, 118, 153, 153, 137, 140, 119,
   125, 69, 74, 10, 68, 29, 34, 13, 103, 87, 65, 74, 77, 133, 111, 218, 195, 146,
   185, 215, 215, 100, 148, 127, 219, 13, 63, 38, 16, 16, 58, 20, 34, 77, 143,
   218, 208, 172, 166, 215, 207, 31, 0, 120, 102, 124, 167, 62, 152, 95, 103,
   146, 154, 10, 140, 156, 133, 75, 184, 186, 117, 5, 20, 86, 203, 8, 214,
   160, 150, 203, 162, 232, 175, 12, 56, 52, 72, 157, 150, 182, 195, 228, 118,
   74, 189, 202, 153, 81, 58, 62, 73, 74, 72, 13, 71, 52, 14, 15, 41, 47, 86, 138,
   197, 188, 175, 145, 216, 105, 156, 144, 195, 178, 92, 98, 156, 17, 55, 34,
   20, 10, 8, 30, 41, 41, 41, 96, 123, 147, 210, 189, 225, 220, 0, 50, 203, 181,
   193, 223, 89, 98, 134, 57, 174, 190, 162, 169, 62, 44, 90, 49, 24, 23, 87,
   246, 86, 165, 192, 234, 216, 191, 193, 232, 88, 32, 59, 68, 133, 184, 170,
   183, 233, 250, 114, 13, 178, 195, 173, 165, 108, 37, 4, 87, 71, 28, 51, 72,
   13, 7, 101, 48, 140, 96, 177, 209, 194, 195, 251, 43, 94, 225, 163, 215,
   183, 31, 19, 35, 27, 32, 18, 16, 60, 19, 48, 42, 125, 90, 148, 111, 198, 186,
   162, 175, 69, 0, 249, 201, 214, 239, 168, 145, 143, 75, 30, 219, 208, 189,
   12, 64, 49, 107, 100, 59, 58, 227, 124, 49, 132, 126, 174, 214, 0, 116,
   113, 14, 116, 138, 88, 94, 116, 125, 88, 114, 255, 132, 208, 215, 160,
   0, 9, 0, 28, 82, 72, 22, 40, 62, 9, 8, 106, 91, 132, 95, 71, 221, 218, 187,
   201, 230, 235, 177, 222, 192, 133, 20, 21, 24, 31, 12, 23, 0, 91, 33, 43, 58,
   191, 94, 165, 58, 172, 170, 137, 0, 132, 171, 190, 199, 70, 142, 252,
   194, 131, 180, 238, 156, 215, 207, 3, 87, 112, 84, 18, 159, 219, 162, 154,
   99, 80, 188, 34, 200, 0, 203, 200, 106, 82, 141, 97, 63, 63, 104, 14, 20,
   221, 189, 222, 156, 236, 36, 0, 2, 25, 91, 85, 23, 29, 43, 48, 124, 69, 141,
   117, 99, 90, 186, 19, 152, 223, 201, 174, 188, 163, 210, 0, 20, 32, 18,
   33, 39, 37, 22, 12, 37, 34, 152, 177, 201, 119, 46, 134, 128, 151, 181, 181,
   194, 232, 168, 72, 6, 172, 194, 108, 125, 142, 179, 172, 225, 130, 41, 57, 3,
   159, 140, 187, 201, 110, 175, 87, 62, 55, 207, 232, 168, 160, 110, 50, 55, 90,
   106, 140, 65, 19, 100, 206, 204, 184, 94, 173, 254, 58, 39, 25, 81, 56,
   35, 33, 44, 122, 120, 116, 116, 96, 93, 22, 79, 71, 151, 202, 231, 199, 220,
   135, 209, 58, 0, 20, 43, 57, 56, 50, 28, 41, 38, 31, 49, 79, 17, 63, 81, 95, 144,
   58, 142, 164, 192, 175, 211, 95, 200, 207, 203, 135, 130, 160, 107, 42,
   198, 243, 69, 6, 47, 80, 125, 243, 162, 62, 5, 31, 4, 71, 174, 158, 185, 161,
   101, 109, 116, 116, 137, 88, 21, 100, 18, 137, 244, 70, 31, 76, 212,
   121, 121, 89, 54, 38, 44, 49, 30, 120, 123, 56, 37, 45, 97, 67, 34, 20, 25,
   215, 197, 75, 90, 197, 141, 113, 29, 43, 30, 16, 36, 9, 33, 16, 26, 37, 20,
   21, 50, 66, 41, 9, 29, 108, 109, 182, 166, 159, 150, 236, 170, 171, 142, 115,
   103, 27, 43, 36, 35, 212, 120, 8, 57, 74, 36, 250, 8, 5, 17, 11, 27, 45, 150,
   155, 217, 80, 99, 109, 138, 171, 84, 7, 66, 27, 49, 70, 113, 39, 39, 39, 116,
   221, 67, 70, 55, 73, 64, 56, 60, 120, 119, 63, 29, 17, 95, 111, 0, 26, 15, 30,
   168, 46, 97, 50, 188, 121, 92, 11, 20, 26, 24, 21, 13, 19, 7, 54, 26, 32, 48,
   51, 48, 24, 57, 3, 18, 255, 236, 182, 203, 146, 233, 171, 149, 86, 11, 2, 231,
   200, 0, 180, 0, 41, 63, 59, 27, 7, 34, 41, 3, 18, 59, 35, 116, 128, 191, 45,
   80, 105, 172, 146, 32, 135, 75, 48, 53, 74, 35, 34, 65, 61, 27, 95, 18, 62,
   68, 65, 53, 58, 62, 104, 153, 66, 4, 9, 4, 93, 58, 5, 14, 73, 116, 82, 32,
   201, 136, 139, 103, 5, 19, 57, 13, 19, 22, 11, 16, 47, 19, 15, 61, 86, 5, 16,
   68, 13, 58, 241, 222, 253, 183, 104, 250, 206, 195, 128, 14, 160, 154, 175,
   15, 71, 34, 37, 81, 29, 55, 22, 52, 8, 12, 23, 9, 30, 67, 79, 91, 45, 63, 146,
   136, 8, 115, 122, 33, 57, 44, 45, 44, 12, 24, 39, 118, 5, 150, 61, 52, 42, 44,
   34, 55, 149, 152, 16, 28, 13, 3, 4, 42, 6, 26, 66, 92, 105, 137, 221, 194,
   99, 123, 90, 5, 31, 21, 3, 10, 24, 12, 21, 22, 33, 70, 0, 2, 44, 9, 17, 17,
   85, 179, 82, 12, 20, 93, 188, 185, 83, 26, 174, 130, 213, 237, 86, 186,
   132, 86, 111, 225, 102, 7, 11, 14, 15, 58, 89, 27, 63, 155, 43, 99, 98, 17, 106,
   78, 48, 62, 49, 29, 23, 56, 218, 175, 184, 47, 94, 32, 41, 44, 42, 55, 46, 34,
   152, 150, 25, 13, 21, 16, 15, 6, 10, 27, 79, 82, 83, 33, 98, 155, 115, 106,
   32, 28, 26, 17, 36, 4, 17, 11, 15, 126, 226, 111, 188, 23, 29, 53, 40, 29,
   29, 88, 30, 214, 56, 1, 0, 145, 182, 169, 170, 170, 173, 78, 80, 64, 212,
   123, 61, 147, 221, 21, 9, 0, 84, 123, 40, 49, 87, 59, 62, 65, 31, 58, 109, 87,
   32, 27, 27, 35, 39, 122, 81, 48, 15, 51, 34, 39, 52, 41, 46, 39, 44, 51, 144,
   150, 34, 32, 10, 3, 11, 11, 1, 46, 81, 72, 105, 62, 70, 5, 36, 132, 119, 7, 32,
   9, 9, 5, 22, 6, 41, 201, 56, 221, 253, 222, 3, 14, 18, 19, 87, 46, 22, 178,
   247, 39, 10, 85, 211, 149, 177, 206, 162, 158, 139, 206, 209, 69, 35, 197,
   161, 184, 212, 176, 6, 47, 31, 33, 51, 24, 41, 27, 62, 75, 63, 59, 87, 81, 80,
   60, 61, 39, 40, 34, 35, 30, 22, 26, 33, 22, 48, 39, 25, 34, 51, 68, 10, 10, 20,
   6, 6, 9, 3, 10, 59, 52, 184, 54, 71, 59, 142, 87, 25, 9, 12, 29, 43, 61, 13, 1,
   70, 185, 209, 118, 236, 245, 250, 1, 2, 83, 60, 28, 17, 49, 168, 0, 6, 132,
   108, 114, 73, 219, 195, 203, 178, 204, 213, 140, 0, 211, 175, 232, 214,
   245, 63, 38, 25, 38, 25, 18, 20, 68, 68, 29, 33, 30, 34, 54, 74, 106, 87, 97, 41,
   24, 26, 37, 17, 86, 90, 65, 30, 49, 30, 40, 70, 72, 98, 13, 19, 18, 18, 8, 9, 7,
   3, 8, 140, 108, 26, 9, 3, 0, 13, 17, 17, 37, 102, 65, 10, 31, 37, 40, 255, 255,
   215, 167, 242, 248, 178, 209, 173, 9, 71, 177, 0, 8, 154, 3, 192, 241, 55,
   227, 191, 152, 216, 227, 185, 199, 83, 231, 207, 169, 233, 216, 140, 6, 47,
   31, 7, 43, 62, 64, 55, 35, 26, 24, 24, 34, 52, 22, 44, 46, 76, 82, 39, 32, 82,
   102, 107, 86, 91, 50, 36, 37, 72, 73}, 
  {236,222, 180, 150, 169, 149, 187, 212, 220, 170, 179, 189, 190, 241, 226, 201, 193, 189,
   216, 231, 158, 147, 12, 0, 31, 62, 67, 73, 13, 34, 9, 0, 0, 3, 24, 48, 28, 147, 211,
   159, 169, 112, 101, 121, 94, 75, 41, 5, 32, 63, 61, 37, 31, 19, 25, 25, 4, 23, 27, 7,
   5, 12, 21, 39, 46, 50, 27, 10, 0, 21, 20, 0, 43, 39, 27, 0, 0, 0, 16, 0, 9, 3, 0, 1,
   13, 9, 15, 28, 22, 15, 185, 204, 107, 104, 185, 142, 175, 254, 188, 99, 151, 213, 222,
   178, 166, 154, 218, 188, 179, 244, 221, 245, 188, 97, 68, 62, 57, 44, 59, 87, 50, 30,
   12, 0, 23, 48, 46, 153, 203, 153, 174, 172, 182, 219, 234, 153, 66, 48, 61, 66, 49, 15,
   21, 51, 39, 28, 8, 32, 21, 0, 0, 10, 25, 34, 42, 52, 31, 14, 0, 2, 20, 14, 46, 53, 42,
   0, 0, 0, 12, 7, 5, 2, 5, 11, 18, 13, 31, 62, 17, 22, 168, 122, 15, 201, 247, 219, 212,
   215, 163, 46, 21, 162, 164, 186, 223, 128, 182, 248, 231, 240, 222, 199, 236, 205, 156,
   88, 120, 107, 66, 73, 87, 77, 32, 21, 32, 46, 56, 85, 98, 88, 166, 178, 173, 190, 206,
   147, 55, 42, 61, 52, 6, 12, 42, 59, 60, 65, 12, 13, 6, 0, 0, 17, 32, 51, 46, 38, 29, 0,
   0, 0, 0, 15, 83, 67, 44, 0, 0, 0, 0, 1, 0, 0, 6, 13, 18, 14, 34, 63, 19, 22, 114, 31,
   57, 249, 237, 189, 231, 224, 64, 18, 8, 54, 39, 107, 166, 85, 54, 217, 217, 228, 219,
   205, 187, 164, 119, 84, 146, 209, 168, 83, 92, 98, 28, 29, 42, 44, 55, 28, 28, 76, 156,
   166, 147, 138, 142, 72, 65, 48, 67, 5, 0, 8, 20, 50, 69, 75, 10, 0, 0, 0, 0, 15, 35,
   145, 142, 39, 19, 0, 0, 0, 0, 0, 105, 95, 64, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 6, 22, 38,
   26, 23, 146, 174, 148, 194, 137, 152, 220, 211, 118, 135, 55, 52, 44, 114, 103, 143,
   119, 76, 104, 255, 242, 255, 191, 182, 186, 170, 77, 190, 220, 223, 173, 109, 21, 40,
   77, 43, 42, 54, 60, 117, 138, 186, 178, 112, 95, 115, 114, 107, 31, 13, 29, 29, 26, 36,
   44, 42, 2, 4, 1, 36, 47, 0, 93, 180, 201, 124, 12, 11, 0, 0, 0, 0, 79, 104, 86, 3, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 9, 19, 50, 41, 162, 226, 231, 137, 67, 129, 235, 204, 160,
   213, 188, 107, 173, 190, 162, 171, 227, 255, 187, 249, 234, 224, 75, 169, 204, 232,
   146, 103, 114, 168, 158, 154, 57, 53, 84, 44, 37, 62, 64, 164, 157, 226, 238, 122, 89,
   107, 120, 104, 100, 109, 73, 29, 27, 12, 26, 65, 73, 95, 37, 59, 128, 80, 103, 188,
   193, 127, 0, 23, 26, 5, 0, 0, 65, 67, 49, 3, 0, 0, 0, 0, 7, 0, 0, 0, 0, 0, 0, 4, 40,
   43, 101, 136, 171, 206, 138, 162, 200, 179, 142, 191, 137, 156, 207, 180, 195, 182, 96,
   171, 230, 170, 178, 205, 128, 107, 172, 206, 165, 138, 80, 59, 116, 150, 85, 61, 57,
   57, 49, 50, 41, 134, 168, 225, 247, 139, 74, 89, 90, 104, 78, 80, 101, 85, 82, 85, 66,
   77, 58, 81, 60, 75, 192, 205, 185, 210, 204, 138, 59, 91, 97, 30, 0, 15, 86, 60, 21,
   11, 32, 17, 7, 23, 16, 0, 0, 4, 10, 3, 3, 0, 43, 45, 15, 119, 197, 175, 194, 182, 169,
   191, 148, 209, 122, 116, 194, 190, 184, 101, 42, 187, 195, 208, 177, 186, 218, 176,
   127, 143, 151, 184, 198, 146, 144, 222, 164, 82, 65, 187, 166, 68, 31, 106, 186, 205,
   215, 207, 178, 103, 80, 115, 79, 109, 123, 87, 52, 81, 85, 64, 89, 76, 84, 31, 100,
   185, 203, 221, 232, 211, 151, 101, 87, 47, 18, 35, 103, 101, 59, 21, 44, 57, 46, 37,
   22, 18, 26, 25, 23, 15, 9, 0, 48, 46, 67, 88, 184, 162, 194, 232, 234, 207, 251, 220,
   76, 62, 156, 238, 252, 204, 144, 196, 208, 230, 128, 81, 83, 99, 155, 176, 215, 193,
   160, 185, 162, 182, 192, 42, 64, 187, 148, 86, 120, 36, 177, 164, 180, 236, 216, 198,
   133, 9, 0, 34, 68, 38, 33, 40, 56, 67, 12, 17, 10, 0, 33, 109, 140, 192, 218, 210, 183,
   140, 113, 93, 56, 29, 83, 89, 92, 80, 70, 73, 81, 42, 43, 36, 38, 37, 37, 36, 25, 23,
   7, 8, 132, 22, 52, 80, 119, 153, 193, 208, 196, 221, 77, 129, 120, 203, 192, 222, 204,
   217, 182, 212, 137, 251, 145, 107, 138, 189, 193, 178, 191, 198, 182, 176, 170, 39, 44,
   140, 84, 97, 138, 67, 113, 164, 186, 210, 241, 236, 204, 51, 0, 24, 76, 84, 57, 15, 71,
   62, 29, 36, 9, 0, 0, 55, 106, 154, 183, 203, 184, 140, 88, 77, 39, 27, 49, 74, 91, 81,
   66, 67, 81, 58, 41, 41, 35, 25, 30, 24, 16, 29, 8, 10, 123, 58, 20, 66, 98, 91, 117,
   197, 199, 192, 40, 162, 249, 206, 222, 189, 209, 214, 136, 151, 117, 99, 219, 143, 130,
   130, 95, 94, 89, 129, 220, 195, 144, 31, 61, 184, 134, 127, 79, 89, 0, 62, 171, 196,
   213, 193, 164, 114, 15, 87, 139, 163, 167, 66, 59, 103, 46, 39, 0, 0, 0, 13, 84, 168,
   207, 178, 105, 45, 15, 0, 10, 7, 45, 45, 52, 61, 65, 65, 71, 57, 31, 11, 17, 12, 3, 6,
   22, 25, 12, 16, 96, 78, 29, 54, 101, 64, 128, 211, 147, 122, 53, 224, 200, 193, 215,
   203, 140, 196, 182, 164, 154, 105, 201, 167, 115, 88, 72, 73, 42, 99, 226, 232, 175,
   29, 108, 196, 188, 194, 158, 153, 105, 115, 177, 186, 182, 198, 175, 151, 162, 109, 72,
   114, 122, 117, 78, 112, 32, 21, 41, 0, 0, 13, 56, 74, 123, 105, 54, 31, 29, 37, 15, 0,
   45, 43, 36, 41, 59, 73, 64, 27, 11, 1, 4, 2, 7, 3, 11, 13, 16, 14, 105, 93, 51, 55, 58,
   3, 74, 158, 33, 80, 95, 210, 131, 0, 147, 190, 174, 185, 221, 209, 229, 255, 221, 155,
   101, 120, 45, 89, 80, 148, 215, 213, 163, 47, 134, 233, 200, 232, 242, 245, 160, 130,
   163, 180, 170, 222, 223, 189, 142, 163, 114, 128, 146, 141, 97, 74, 28, 65, 56, 30, 0,
   45, 64, 37, 62, 84, 38, 23, 24, 39, 33, 0, 23, 40, 37, 22, 33, 71, 68, 27, 5, 0, 1, 1,
   4, 1, 10, 12, 13, 11, 80, 81, 23, 20, 26, 24, 45, 112, 15, 67, 89, 53, 30, 25, 46, 221,
   230, 219, 219, 226, 219, 254, 217, 128, 86, 84, 51, 148, 109, 120, 128, 114, 94, 66,
   123, 213, 208, 216, 243, 221, 173, 110, 131, 149, 78, 139, 245, 204, 189, 247, 173,
   124, 119, 88, 98, 106, 114, 136, 100, 44, 32, 49, 73, 75, 88, 49, 4, 0, 11, 20, 32, 33,
   35, 19, 17, 31, 46, 62, 59, 42, 17, 10, 14, 19, 5, 5, 14, 6, 11, 12, 50, 60, 17, 16,
   33, 18, 23, 69, 0, 23, 85, 23, 49, 20, 31, 184, 248, 240, 230, 207, 63, 177, 198, 229,
   172, 90, 90, 141, 49, 87, 70, 180, 164, 50, 107, 171, 180, 225, 176, 174, 188, 173,
   154, 101, 40, 140, 203, 210, 182, 232, 161, 93, 93, 171, 169, 119, 87, 87, 69, 133,
   139, 93, 138, 168, 132, 48, 0, 0, 10, 10, 22, 54, 91, 34, 7, 38, 53, 60, 59, 63, 25,
   43, 39, 45, 44, 34, 13, 5, 12, 15, 11, 85, 26, 3, 2, 0, 0, 37, 0, 27, 87, 9, 4, 35, 8,
   23, 149, 160, 175, 216, 190, 166, 186, 210, 162, 116, 131, 70, 158, 179, 134, 211, 171,
   44, 135, 213, 234, 228, 200, 211, 116, 190, 195, 187, 86, 156, 187, 237, 229, 225, 207,
   95, 54, 153, 189, 94, 80, 86, 55, 112, 128, 109, 154, 197, 192, 87, 0, 0, 0, 0, 11, 45,
   106, 85, 85, 100, 103, 119, 111, 91, 32, 42, 38, 56, 54, 58, 40, 20, 15, 15, 0, 55, 40,
   8, 13, 13, 35, 21, 11, 0, 102, 4, 0, 34, 63, 0, 200, 248, 208, 231, 183, 136, 171, 84,
   78, 86, 93, 154, 133, 159, 131, 103, 95, 55, 55, 137, 71, 129, 255, 162, 80, 204, 200,
   166, 114, 118, 209, 249, 234, 190, 214, 195, 135, 55, 146, 92, 95, 87, 61, 77, 148,
   133, 144, 213, 216, 157, 93, 80, 0, 24, 24, 33, 90, 136, 159, 171, 197, 223, 202, 207,
   68, 6, 34, 17, 25, 24, 23, 14, 33, 18, 37, 9, 68, 53, 65, 72, 95, 41, 20, 19, 105, 133,
   90, 52, 86, 40, 91, 218, 137, 141, 136, 124, 115, 126, 33, 102, 68, 114, 155, 152, 88,
   0, 15, 88, 98, 192, 174, 215, 236, 222, 162, 187, 192, 137, 107, 71, 111, 237, 217,
   193, 238, 217, 208, 100, 140, 118, 114, 58, 64, 60, 136, 148, 195, 214, 190, 182, 151,
   101, 24, 18, 9, 1, 33, 79, 118, 151, 182, 209, 165, 193, 178, 29, 0, 11, 13, 2, 0, 16,
   12, 10, 19, 18, 46, 32, 56, 74, 55, 60, 12, 48, 82, 119, 74, 80, 99, 30, 43, 79, 127,
   123, 88, 189, 186, 220, 137, 6, 20, 81, 100, 146, 148, 66, 16, 81, 36, 96, 249, 190,
   243, 114, 198, 158, 164, 197, 227, 160, 8, 112, 240, 221, 246, 223, 245, 252, 185, 109,
   127, 60, 57, 59, 154, 212, 193, 200, 197, 130, 63, 28, 19, 40, 126, 129, 114, 113, 125,
   153, 130, 185, 191, 184, 245, 189, 72, 47, 29, 6, 0, 0, 10, 11, 0, 28, 52, 44, 38, 125,
   68, 151, 83, 59, 83, 115, 166, 102, 102, 44, 45, 73, 70, 110, 123, 119, 255, 197, 181,
   227, 62, 20, 58, 0, 33, 73, 16, 32, 27, 0, 34, 203, 176, 68, 218, 85, 141, 188, 109,
   28, 168, 172, 106, 204, 91, 134, 198, 239, 204, 203, 95, 55, 56, 103, 89, 216, 190,
   188, 248, 151, 71, 18, 25, 71, 182, 197, 221, 170, 189, 187, 131, 169, 242, 190, 182,
   170, 116, 62, 60, 35, 11, 11, 8, 5, 54, 46, 62, 64, 55, 108, 94, 160, 140, 158, 111,
   93, 139, 125, 139, 40, 28, 31, 74, 158, 144, 30, 199, 255, 225, 206, 140, 204, 180,
   202, 241, 92, 26, 93, 69, 38, 0, 56, 255, 81, 183, 219, 195, 75, 0, 148, 208, 209, 218,
   214, 226, 124, 186, 220, 158, 74, 126, 132, 99, 78, 62, 99, 82, 118, 145, 139, 111, 58,
   50, 27, 101, 155, 176, 160, 190, 180, 118, 133, 171, 154, 180, 165, 129, 107, 65, 10,
   0, 0, 63, 53, 4, 43, 21, 102, 201, 151, 69, 163, 91, 184, 152, 82, 51, 61, 101, 55, 17,
   58, 160, 166, 114, 136, 211, 180, 170, 198, 101, 180, 228, 249, 175, 32, 69, 91, 52,
   15, 0, 145, 207, 56, 117, 219, 126, 78, 8, 35, 113, 168, 214, 201, 213, 198, 171, 192,
   193, 71, 146, 181, 113, 66, 47, 85, 36, 92, 104, 86, 77, 66, 60, 40, 0, 49, 143, 164,
   168, 97, 98, 56, 99, 97, 94, 141, 94, 111, 49, 6, 7, 6, 38, 29, 4, 37, 5, 43, 233, 203,
   64, 174, 104, 193, 209, 225, 184, 38, 123, 44, 0, 128, 136, 140, 161, 168, 230, 232,
   150, 126, 28, 79, 210, 176, 24, 38, 57, 71, 59, 36, 0, 60, 182, 100, 41, 234, 163, 126,
   3, 77, 38, 12, 8, 18, 170, 178, 158, 155, 142, 198, 22, 119, 132, 120, 79, 23, 10, 149,
   184, 149, 102, 55, 64, 76, 43, 23, 210, 192, 139, 137, 158, 48, 25, 32, 14, 24, 37, 32,
   7, 0, 0, 0, 25, 23, 0, 41, 54, 66, 46, 146, 59, 132, 27, 220, 191, 228, 200, 89, 77,
   61, 0, 151, 165, 174, 88, 161, 208, 116, 44, 11, 28, 57, 197, 255, 155, 16, 31, 40, 62,
   0, 0, 221, 144, 132, 15, 147, 189, 201, 60, 54, 57, 51, 67, 40, 20, 80, 85, 120, 144,
   186, 63, 119, 195, 206, 98, 20, 121, 150, 162, 178, 88, 88, 68, 177, 127, 57, 187, 180,
   97, 118, 170, 139, 45, 5, 19, 0, 4, 0, 0, 0, 0, 0, 21, 18, 0, 13, 111, 112, 48, 114,
   224, 159, 125, 86, 238, 176, 181, 134, 57, 24, 52, 129, 229, 181, 0, 170, 251, 145, 18,
   128, 135, 124, 0, 141, 163, 168, 32, 66, 18, 108, 223, 222, 177, 148, 12, 147, 165,
   255, 239, 144, 209, 199, 48, 234, 65, 77, 73, 44, 181, 173, 243, 168, 224, 244, 150,
   152, 156, 107, 170, 145, 101, 97, 83, 172, 147, 81, 116, 133, 102, 98, 120, 151, 42,
   41, 18, 4, 0, 1, 0, 0, 0, 0, 0, 4, 8, 0, 40, 55, 129, 155, 204, 195, 182, 167, 83, 145,
   255, 111, 251, 68, 25, 105, 214, 147, 75, 188, 185, 138, 0, 176, 205, 167, 201, 166,
   118, 119, 94, 68, 204, 104, 70, 190, 255, 113, 25, 147, 101, 178, 225, 186, 176, 180,
   143, 191, 205, 187, 164, 69, 76, 78, 214, 146, 245, 210, 255, 70, 87, 152, 148, 72,
   112, 70, 111, 137, 136, 68, 58, 50, 72, 63, 149, 155, 67, 28, 39, 29, 2, 0, 0, 0, 0, 0,
   2, 0, 39, 0, 40, 47, 46, 123, 184, 173, 195, 207, 197, 118, 43, 109, 209, 51, 39, 30,
   117, 71, 53, 100, 207, 86, 66, 190, 171, 244, 189, 158, 239, 230, 38, 125, 226, 178,
   121, 174, 50, 127, 0, 125, 248, 185, 191, 192, 93, 58, 145, 103, 138, 187, 120, 73, 29,
   27, 56, 65, 252, 248, 128, 105, 102, 169, 53, 63, 91, 109, 93, 76, 73, 85, 69, 56, 85,
   89, 97, 125, 69, 61, 63, 47, 18, 0, 10, 0, 0, 0, 0, 0, 126, 47, 16, 36, 33, 25, 126,
   172, 149, 198, 191, 102, 30, 195, 255, 62, 44, 44, 28, 121, 11, 0, 83, 7, 80, 75, 255,
   167, 218, 150, 210, 236, 179, 154, 207, 219, 106, 21, 65, 117, 15, 68, 226, 190, 132,
   191, 63, 40, 44, 133, 191, 140, 54, 89, 38, 30, 63, 64, 117, 164, 117, 118, 218, 175,
   141, 85, 97, 82, 53, 41, 43, 20, 77, 70, 113, 116, 77, 63, 68, 44, 37, 15, 10, 6, 36,
   35, 14, 21, 0, 0, 184, 138, 48, 54, 33, 38, 34, 123, 206, 174, 255, 43, 82, 237, 121,
   62, 36, 56, 17, 29, 82, 16, 141, 81, 37, 33, 100, 221, 192, 181, 231, 255, 217, 217,
   223, 230, 119, 82, 69, 135, 43, 92, 240, 199, 75, 91, 47, 93, 63, 190, 188, 155, 24,
   65, 33, 48, 62, 33, 87, 131, 89, 161, 218, 213, 130, 91, 91, 47, 88, 63, 49, 12, 37,
   53, 105, 92, 105, 53, 101, 17, 28, 6, 5, 11, 41, 40, 17, 30, 25, 24, 118, 76, 16, 41,
   26, 40, 20, 155, 164, 215, 70, 36, 174, 184, 161, 56, 31, 10, 24, 31, 94, 110, 64, 49,
   52, 127, 102, 134, 197, 187, 212, 146, 248, 223, 181, 200, 187, 174, 99, 53, 116, 102,
   120, 147, 69, 18, 121, 99, 26, 103, 145, 65, 0, 23, 30, 75, 72, 23, 66, 104, 82, 169,
   211, 184, 109, 118, 29, 42, 66, 34, 25, 3, 20, 38, 72, 93, 57, 59, 41, 59, 0, 0, 0, 7,
   44, 43, 19, 44, 11, 0, 66, 44, 0, 26, 43, 35, 54, 224, 163, 255, 0, 43, 255, 176, 142,
   114, 8, 11, 0, 9, 78, 75, 63, 68, 81, 106, 123, 187, 166, 249, 255, 255, 235, 212, 255,
   250, 193, 187, 108, 81, 97, 68, 171, 181, 0, 163, 90, 132, 20, 29, 99, 78, 0, 52, 76,
   121, 8, 62, 120, 88, 69, 22, 147, 212, 223, 185, 28, 0, 6, 9, 27, 14, 45, 105, 137,
   112, 76, 40, 35, 9, 7, 1, 0, 7, 36, 36, 6, 16, 0, 0, 72, 94, 7, 20, 30, 29, 61, 7, 0,
   4, 18, 97, 146, 117, 136, 48, 37, 24, 0, 46, 114, 99, 50, 42, 40, 121, 12, 97, 147,
   112, 77, 177, 178, 60, 114, 205, 114, 198, 148, 120, 91, 5, 190, 196, 103, 159, 89,
   164, 65, 0, 46, 111, 82, 25, 54, 105, 13, 146, 82, 130, 27, 0, 0, 189, 224, 191, 27, 0,
   6, 76, 58, 3, 43, 77, 150, 159, 161, 128, 69, 1, 12, 0, 0, 2, 22, 38, 18, 8, 0, 0, 49,
   9, 54, 0, 10, 34, 37, 29, 0, 0, 0, 126, 198, 90, 87, 0, 50, 0, 15, 45, 63, 61, 35, 79,
   0, 0, 0, 11, 67, 48, 30, 178, 177, 144, 146, 217, 238, 208, 184, 155, 105, 70, 206,
   235, 227, 229, 192, 202, 165, 192, 4, 133, 98, 9, 54, 1, 100, 183, 99, 56, 3, 68, 102,
   43, 163, 230, 153, 71, 19, 41, 60, 4, 36, 79, 118, 143, 204, 173, 90, 12, 4, 0, 0, 11,
   30, 42, 5, 0, 0, 0, 80, 40, 0, 15, 0, 0, 67, 18, 18, 9, 0, 124, 42, 77, 68, 1, 58, 0,
   23, 20, 13, 82, 0, 180, 96, 0, 8, 4, 54, 30, 17, 255, 196, 118, 210, 221, 204, 229,
   153, 98, 143, 250, 190, 110, 242, 239, 215, 184, 102, 107, 134, 67, 62, 119, 0, 5, 164,
   128, 66, 23, 0, 25, 134, 127, 160, 171, 156, 148, 90, 25, 52, 50, 140, 140, 169, 131,
   194, 212, 182, 72, 7, 0, 0, 0, 12, 15, 0, 0, 0, 0, 53, 80, 0, 29, 38, 30, 53, 44, 38,
   0, 6, 4, 29, 54, 11, 80, 80, 7, 23, 17, 1, 33, 40, 150, 162, 0, 4, 0, 32, 84, 84, 90,
   146, 169, 168, 177, 128, 189, 179, 104, 0, 225, 227, 234, 29, 0, 194, 236, 111, 83, 32,
   154, 115, 196, 0, 59, 50, 14, 10, 70, 11, 49, 40, 207, 229, 154, 52, 51, 44, 0, 17, 81,
   68, 114, 47, 39, 150, 180, 175, 187, 59, 0, 0, 7, 23, 38, 38, 82, 0, 4, 79, 99, 60, 0,
   44, 121, 153, 52, 30, 31, 0, 0, 74, 95, 5, 21, 118, 51, 0, 12, 37, 55, 15, 10, 4, 31,
   18, 51, 66, 11, 119, 160, 204, 58, 254, 210, 103, 0, 0, 1, 83, 55, 98, 63, 228, 160,
   195, 58, 126, 70, 217, 15, 98, 26, 92, 59, 51, 25, 47, 31, 94, 94, 130, 148, 115, 181,
   209, 85, 77, 68, 0, 5, 44, 136, 152, 207, 149, 160, 170, 151, 120, 0, 6, 5, 6, 58, 79,
   95, 0, 3, 111, 85, 62, 131, 4, 45, 56, 71, 28, 133, 98, 42, 42, 40, 10, 187, 248, 141,
   2, 5, 42, 33, 118, 139, 68, 63, 51, 0, 60, 27, 74, 139, 164, 76, 100, 246, 113, 0, 0,
   0, 51, 108, 68, 62, 213, 195, 36, 170, 248, 138, 132, 13, 2, 120, 101, 62, 103, 53, 41,
   34, 74, 56, 0, 90, 110, 142, 198, 90, 0, 21, 45, 20, 0, 21, 0, 20, 16, 104, 158, 160,
   84, 0, 0, 0, 9, 48, 88, 114, 69, 63, 102, 93, 12, 124, 130, 26, 19, 4, 133, 134, 135,
   112, 50, 89, 124, 129, 212, 66, 7, 0, 52, 97, 103, 217, 146, 23, 120, 0, 2, 95, 41, 86,
   32, 122, 38, 48, 118, 0, 0, 32, 16, 68, 86, 142, 239, 0, 75, 194, 163, 181, 86, 12, 66,
   113, 59, 165, 48, 58, 24, 40, 41, 23, 30, 151, 82, 105, 194, 14, 194, 113, 42, 29, 31,
   13, 8, 55, 66, 83, 130, 174, 108, 0, 0, 0, 0, 24, 68, 75, 56, 40, 112, 5, 33, 20, 137,
   157, 48, 46, 167, 202, 162, 205, 185, 157, 135, 104, 169, 43, 29, 16, 78, 116, 77, 94,
   65, 14, 29, 43, 31, 138, 11, 39, 48, 111, 98, 52, 5, 34, 27, 72, 37, 42, 70, 12, 46,
   173, 59, 15, 14, 138, 11, 56, 114, 125, 120, 126, 73, 159, 15, 14, 22, 69, 72, 108, 16,
   56, 111, 56, 210, 183, 110, 57, 39, 0, 0, 29, 38, 34, 78, 156, 169, 62, 26, 0, 0, 5,
   25, 37, 40, 44, 106, 0, 63, 72, 53, 157, 86, 53, 0, 205, 240, 217, 199, 250, 137, 136,
   151, 51, 30, 0, 24, 48, 136, 42, 26, 0, 0, 0, 26, 23, 67, 23, 127, 155, 115, 20, 37,
   110, 65, 149, 86, 25, 35, 71, 75, 75, 44, 98, 145, 17, 47, 64, 128, 98, 84, 101, 91, 0,
   28, 2, 50, 59, 63, 58, 44, 13, 27, 100, 125, 140, 121, 82, 22, 0, 0, 16, 23, 23, 9, 63,
   73, 24, 13, 24, 44, 0, 0, 16, 33, 55, 132, 225, 62, 0, 31, 14, 21, 41, 73, 25, 201,
   168, 146, 138, 253, 157, 182, 98, 109, 123, 67, 0, 93, 48, 23, 172, 144, 75, 7, 1, 60,
   210, 140, 124, 33, 168, 219, 209, 193, 71, 20, 46, 81, 60, 93, 79, 30, 45, 107, 181,
   34, 88, 6, 0, 103, 70, 83, 21, 69, 43, 33, 46, 74, 54, 78, 119, 71, 0, 52, 113, 167,
   37, 94, 117, 64, 15, 28, 64, 104, 12, 12, 0, 1, 33, 37, 1, 10, 30, 0, 0, 77, 114, 168,
   0, 87, 93, 0, 7, 138, 170, 177, 190, 53, 59, 70, 209, 163, 61, 37, 77, 80, 8, 45, 102,
   0, 67, 198, 182, 135, 0, 147, 147, 163, 153, 12, 197, 216, 195, 186, 104, 48, 44, 51,
   64, 86, 145, 105, 57, 17, 93, 21, 89, 0, 62, 68, 62, 68, 20, 81, 46, 14, 49, 13, 46,
   36, 30, 49, 66, 10, 74, 100, 148, 200, 135, 82, 54, 90, 153, 125, 123, 50, 0, 5, 32,
   14, 0, 0, 25, 0, 0, 86, 37, 108, 0, 166, 192, 188, 154, 155, 142, 162, 216, 105, 102,
   166, 206, 228, 111, 86, 28, 36, 41, 26, 92, 41, 0, 118, 207, 175, 176, 162, 86, 220,
   58, 45, 153, 133, 202, 215, 181, 15, 51, 40, 44, 34, 69, 104, 64, 89, 37, 21, 45, 0,
   51, 145, 192, 228, 32, 34, 18, 0, 0, 87, 89, 69, 54, 24, 0, 0, 56, 139, 173, 212, 192,
   121, 56, 90, 58, 97, 88, 61, 0, 35, 77, 42, 3, 0, 30, 0, 0, 27, 15, 0, 0, 43, 157, 200,
   247, 227, 131, 84, 180, 250, 255, 166, 236, 139, 55, 7, 24, 18, 29, 9, 31, 51, 174,
   164, 164, 222, 183, 208, 242, 45, 23, 8, 26, 154, 160, 139, 0, 68, 6, 61, 35, 49, 98,
   14, 35, 29, 24, 78, 0, 0, 82, 255, 198, 101, 112, 186, 167, 58, 65, 191, 143, 77, 26,
   40, 19, 28, 21, 72, 197, 164, 204, 228, 99, 78, 90, 54, 43, 17, 0, 5, 77, 35, 12, 0,
   14, 0, 0, 48, 114, 13, 51, 0, 11, 132, 194, 244, 211, 190, 197, 226, 205, 114, 199,
   194, 33, 32, 12, 17, 74, 44, 36, 143, 169, 149, 159, 196, 120, 165, 231, 32, 33, 23,
   105, 101, 128, 117, 0, 65, 172, 122, 79, 110, 0, 0, 76, 32, 25, 63, 31, 153, 227, 36,
   0, 9, 67, 154, 97, 223, 70, 192, 148, 89, 0, 34, 80, 28, 55, 81, 40, 194, 186, 192, 84,
   94, 134, 151, 121, 101, 64, 75, 27, 60, 14, 60, 33, 20, 0, 104, 85, 70, 105, 79, 142,
   88, 215, 196, 144, 190, 246, 217, 109, 125, 124, 243, 0, 37, 0, 8, 47, 54, 55, 58, 64,
   117, 165, 200, 203, 162, 250, 212, 51, 27, 137, 81, 71, 123, 38, 157, 115, 131, 163,
   133, 0, 96, 132, 108, 36, 164, 186, 118, 18, 6, 129, 178, 0, 194, 160, 151, 216, 148,
   255, 165, 0, 26, 67, 102, 174, 173, 180, 185, 204, 88, 89, 219, 219, 176, 79, 36, 14,
   16, 12, 37, 2, 54, 53, 0, 0, 56, 20, 83, 155, 223, 225, 205, 159, 231, 78, 153, 161,
   221, 215, 122, 112, 155, 0, 51, 21, 31, 63, 20, 49, 40, 4, 37, 83, 134, 200, 222, 208,
   238, 206, 12, 54, 207, 193, 180, 238, 102, 84, 147, 61, 178, 202, 149, 184, 61, 33, 77,
   35, 35, 81, 115, 255, 85, 154, 179, 220, 227, 249, 221, 255, 94, 0, 13, 66, 145, 193,
   193, 214, 239, 206, 68, 11, 190, 204, 196, 196, 152, 47, 0, 38, 31, 0, 0, 30, 0, 0, 56,
   0, 175, 123, 202, 217, 233, 213, 206, 0, 129, 252, 188, 223, 222, 49, 22, 0, 0, 5, 0,
   41, 101, 47, 51, 0, 96, 63, 123, 136, 239, 214, 184, 184, 82, 20, 255, 223, 237, 255,
   190, 154, 156, 97, 53, 241, 231, 236, 14, 62, 35, 83, 128, 91, 81, 238, 126, 47, 118,
   102, 202, 246, 21, 127, 108, 0, 99, 150, 92, 93, 118, 155, 83, 67, 241, 144, 212, 214,
   162, 25, 16, 0, 0, 22, 30, 0, 0, 8, 0, 0, 50, 36, 118, 111, 100, 239, 255, 163, 145,
   175, 221, 193, 251, 210, 176, 0, 0, 4, 5, 0, 14, 56, 86, 46, 22, 38, 165, 71, 156, 117,
   167, 183, 162, 7, 151, 141, 201, 233, 88, 166, 255, 202, 150, 150, 249, 190, 233, 231,
   0, 76, 117, 106, 55, 155, 241, 190, 148, 88, 85, 210, 71, 196, 20, 231, 225, 61, 42,
   93, 74, 101, 87, 127, 39, 0, 181, 141, 199, 194, 255, 59, 0, 0, 0, 54, 31, 0, 0, 0, 48,
   95, 28, 97, 92, 114, 130, 188, 41, 173, 182, 157, 149, 203, 203, 212, 20, 41, 5, 2, 23,
   22, 13, 40, 0, 45, 7, 136, 167, 184, 95, 64, 121, 136, 164, 201, 205, 178, 237, 177,
   84, 27, 185, 214, 132, 109, 147, 188, 184, 246, 139, 37, 53, 10, 177, 123, 199, 225,
   119, 171, 83, 69, 73, 190, 244, 192, 196, 90, 25, 15, 58, 108, 127, 44, 55, 80, 181,
   164, 152, 96, 160, 233, 36, 0, 0, 32, 0, 0, 0, 0, 86, 83, 108, 102, 65, 74, 32, 46, 57,
   184, 194, 217, 168, 201, 145, 176, 44, 30, 6, 26, 40, 37, 24, 19, 29, 17, 17, 32, 62,
   0, 37, 72, 83, 123, 45, 157, 182, 187, 179, 206, 111, 230, 194, 218, 153, 125, 164,
   102, 58, 228, 242, 53, 0, 36, 83, 111, 255, 208, 61, 0, 7, 0, 74, 160, 183, 231, 163,
   79, 77, 80, 80, 112, 65, 0, 102, 0, 105, 208, 34, 6, 53, 179, 94, 72, 43, 22, 3, 12,
   27, 9, 66, 59, 27, 34, 37, 93, 96, 50, 23, 40, 186, 194, 67, 86, 226, 157, 116, 44, 33,
   13, 0, 18, 0, 6, 11, 0, 27, 3, 0, 32, 41, 14, 4, 0, 75, 101, 185, 162, 166, 161, 255,
   215, 138, 134, 118, 99, 34, 54, 58, 80, 255, 150, 10, 62, 68, 0, 228, 0, 57, 47, 13,
   32, 39, 111, 133, 206, 38, 64, 69, 108, 137, 47, 0, 43, 0, 14, 30, 83, 5, 2, 22, 93,
   190, 15, 18, 9, 15, 0, 0, 0, 59, 55, 48, 27, 12, 84, 115, 18, 15, 0, 15, 166, 41, 86,
   54, 209, 110, 48, 0, 3, 9, 17, 0, 0, 20, 0, 31, 9, 15, 41, 25, 8, 25, 48, 0, 0, 250,
   227, 195, 236, 160, 255, 152, 125, 79, 2, 15, 255, 214, 41, 230, 30, 51, 88, 69, 3, 0,
   15, 91, 37, 28, 84, 45, 92, 104, 172, 0, 56, 73, 149, 117, 0, 103, 39, 0, 29, 42, 12,
   5, 16, 29, 0, 77, 0, 22, 32, 16, 0, 4, 5, 82, 105, 79, 0, 0, 0, 90, 99, 28, 13, 86,
   109, 67, 21, 198, 177, 162, 102, 0, 2, 23, 5, 4, 3, 15, 18, 30, 2, 0, 53, 71, 0, 20,
   70, 20, 41, 229, 213, 255, 223, 99, 255, 213, 178, 116, 5, 171, 194, 170, 40, 112, 68,
   49, 122, 54, 47, 0, 27, 49, 46, 35, 50, 55, 59, 56, 66, 0, 43, 119, 116, 0, 66, 82, 0,
   5, 24, 18, 24, 0, 0, 0, 82, 0, 87, 3, 8, 0, 0, 0, 11, 96, 103, 23, 14, 5, 6, 1, 72, 3,
   10, 73, 78, 97, 140, 218, 224, 96, 107, 83, 0, 0, 4, 0, 19, 19, 14, 14, 5, 0, 53, 0,
   11, 39, 11, 32, 11, 64, 172, 93, 44, 16, 115, 203, 179, 62, 19, 185, 162, 209, 255,
   126, 218, 143, 134, 140, 224, 78, 0, 51, 46, 26, 106, 118, 26, 39, 128, 0, 69, 70, 0,
   84, 40, 9, 32, 0, 0, 0, 31, 196, 137, 145, 17, 59, 0, 0, 0, 0, 0, 0, 0, 90, 82, 21, 10,
   19, 14, 8, 42, 20, 39, 75, 79, 81, 31, 91, 191, 125, 118, 16, 14, 0, 4, 24, 12, 0, 25,
   0, 112, 177, 98, 176, 31, 11, 67, 44, 24, 0, 85, 41, 239, 66, 34, 4, 140, 147, 166,
   181, 195, 183, 111, 120, 96, 220, 168, 85, 146, 197, 0, 49, 31, 92, 168, 64, 48, 63,
   33, 5, 35, 4, 29, 85, 49, 0, 0, 0, 5, 12, 93, 57, 10, 0, 15, 0, 0, 0, 0, 0, 0, 0, 0,
   80, 85, 0, 0, 0, 19, 40, 29, 44, 22, 25, 17, 91, 78, 99, 23, 79, 108, 98, 0, 6, 0, 0,
   64, 17, 19, 20, 181, 30, 198, 244, 255, 0, 27, 43, 27, 106, 16, 33, 212, 255, 63, 35,
   93, 230, 119, 188, 240, 180, 182, 133, 195, 214, 91, 72, 193, 183, 212, 206, 165, 11,
   69, 68, 29, 73, 52, 66, 0, 22, 27, 40, 97, 111, 104, 105, 15, 21, 0, 17, 72, 59, 53,
   22, 0, 0, 0, 0, 0, 0, 0, 51, 39, 0, 0, 0, 21, 46, 11, 25, 31, 18, 8, 159, 69, 111, 61,
   164, 108, 0, 0, 2, 12, 19, 79, 0, 9, 43, 169, 199, 101, 212, 255, 237, 9, 15, 103, 84,
   12, 22, 58, 180, 19, 19, 152, 132, 98, 78, 228, 207, 224, 187, 200, 209, 147, 16, 194,
   187, 255, 223, 241, 59, 45, 43, 21, 37, 42, 56, 48, 43, 0, 1, 32, 21, 33, 110, 86, 62,
   57, 9, 26, 13, 16, 0, 38, 37, 16, 0, 0, 0, 0, 34, 35, 90, 0, 0, 0, 28, 0, 0, 40, 0, 0,
   109, 89, 36, 0, 0, 31, 0, 0, 0, 18, 76, 56, 0, 10, 23, 23, 243, 239, 189, 158, 230,
   227, 165, 224, 191, 4, 75, 172, 16, 38, 141, 18, 210, 236, 59, 222, 207, 182, 215, 211,
   161, 188, 86, 217, 232, 215, 232, 200, 116, 0, 50, 17, 32, 89, 64, 42, 23, 0, 0, 0, 1,
   1, 54, 0, 12, 10, 40, 57, 16, 0, 55, 53, 61, 54, 56, 18, 14, 16, 18, 9}
};

int out_width = 90;
int out_length = 59;
